package physics;

import java.util.*;

public class Analyzer {
	static public HashSet<String> bounded_variables = new HashSet();
	public static void analyze(){
		bounded_variables.clear();
		
		update_mass_on_slope();
		update_mass_wire_connections();
		update_pulley_connections();
		
		analyze_tags();
		analyze_angles();
		analyze_connectivity();
		analyze_gaps();
		analyze_friction();
	}
	
	public static void update_pulley_connections(){
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof Pulley){
				((Pulley)visual).update_wires();
			}
		}
	}
	public static void update_mass_wire_connections(){
		for(Mass mass : MainPanel.mainPanel.masses()){
			for(Wire wire : MainPanel.mainPanel.wires()){
				if (Vector2D.distance(mass.v0, wire.v0) < Mass.r+0.1){
					wire.v0.warp(mass.v0);
				}
				else if (Vector2D.distance(mass.v0, wire.v1) < Mass.r+0.1){
					wire.v1.warp(mass.v0);
				}
			}
		}
	}
	public static void update_mass_on_slope(){
		for(Mass mass : MainPanel.mainPanel.masses()){
			for(Wall wall : MainPanel.mainPanel.walls()){
				if (wall.v0.x == wall.v1.x || wall.v0.y == wall.v1.y)
					continue;
				if (wall.get_distance(mass.v0) < mass.r+5){
					Vector2D vec = new Vector2D(wall.v0, mass.v0);
					Vector2D tangent = new Vector2D(wall.v0, wall.v1);
					tangent.normalize();
					Vector2D normal = Vector2D.rotate90(tangent);
					Vector2D foot = Vector2D.add(wall.v0, Vector2D.multiply(tangent, Vector2D.dot_product(tangent,  vec)));
					if (Vector2D.dot_product(normal, new Vector2D(foot, mass.v0))<0)
						normal = Vector2D.multiply(normal,  -1);
					Vector2D new_position = Vector2D.add(foot,  Vector2D.multiply(normal,  Mass.r));
					mass.v0.warp(new_position);
				}
			}
		}
	}
	

	/*
	public static void compute_forces(){
		ArrayList<Visual> visuals = MainPanel.mainPanel.visuals;
		
		for(Visual visual : visuals){
			if (visual instanceof Mass){
				visual.set_property("force", "m * g");
			}
		}		
		
		
	}
	*/
	public static void analyze_gaps(){
		MainPanel.mainPanel.gap_labels.clear();
		ArrayList<Visual> visuals = MainPanel.mainPanel.visuals;
		for(Visual visual : visuals){
			if (visual instanceof Gap){
				Gap gap = (Gap) visual;
				Bucket bucket0 = get_nearest_bucket(gap.v0);
				Bucket bucket1 = get_nearest_bucket(gap.v1);
				if (bucket0 == null || bucket1 == null)
					continue;
				for(Node node0 : bucket0.nodes){
					for(Node node1 : bucket1.nodes){
						add_gap_label(node0, node1, insert_multiplier(gap.label));
						add_gap_label(node1, node0, insert_multiplier(gap.label));
					}
				}
				
				bounded_variables.add(remove_numeric_part(gap.label));
			}
		}

		for(Visual visual : visuals){
			if (visual instanceof Stick){
				Stick stick = (Stick)  visual;
				
				String length_label = Analyzer.get_gap_label(stick.start_node,  stick.end_node);
				if ( length_label != null){
					stick.set_property("length", length_label, true, true);
				}
				else {
					add_gap_label(stick.start_node,  stick.end_node,  stick.get_property_value("length"));
					add_gap_label(stick.end_node,  stick.start_node,  stick.get_property_value("length"));
				}
			}
		}
		
	}
	public static void analyze_angles(){
		ArrayList<Visual> visuals = MainPanel.mainPanel.visuals;

		for(Visual visual : visuals){
			if (visual instanceof EdgeVisual){
				((EdgeVisual) visual).set_default_angle();
			}
		}
		
		angle_equations.clear();
		
		for(Visual visual : visuals){
			if (visual instanceof Arc){
				Arc arc = (Arc) visual;
				if (arc.label.length() ==0)
					continue;
				
				Vector2D vec0 = new Vector2D(arc.center, arc.v0);
				Vector2D vec1 = new Vector2D(arc.center, arc.v1);
				EdgeVisual visual0 = pick_edge_visual(arc.v0);
				EdgeVisual visual1 = pick_edge_visual(arc.v1);
				
				int angle0_onscreen = (int) Vector2D.get_angle_360(new Vector2D(1,0), vec0);
				int angle1_onscreen = (int) Vector2D.get_angle_360(new Vector2D(1,0), vec1);
				
				
				String angle0 = (visual0 != null) ? get_angle(visual0, arc.center) : "" + angle0_onscreen + " * Pi / 180 ";
				String angle1 = (visual1 != null) ? get_angle(visual1, arc.center) : "" + angle1_onscreen + " * Pi / 180 ";
				
				String label = arc.label;
				if (is_number(label))
					label = convert_to_radian(label);
				
				int sign = 1;
				if (angle1_onscreen < angle0_onscreen) 
					sign *= -1;
				if (Math.abs(angle1_onscreen - angle0_onscreen) > 180)
					sign *= -1;
						
				String equation = (sign > 0) ?
						label + " = " + angle1 + " - " + angle0 
						:
						label  + " = " +angle0 + " - " + angle1;
				
				angle_equations.add(equation);
				
				
				if (visual0 != null && visual0.get_variable("angle").startsWith("angle"))
					change_constant_property_free(visual0, "angle");
				if (visual1 != null && visual1.get_variable("angle").startsWith("angle"))
					change_constant_property_free(visual1, "angle");
				
				bounded_variables.add(arc.label);

			}
		}
		
		// couple parallel pairs on the screen
		ArrayList<EdgeVisual> edgeVisuals = MainPanel.mainPanel.wires_and_walls_and_arrows();
		for(int i=0; i<edgeVisuals.size()-1; i++){
			EdgeVisual edge0 = edgeVisuals.get(i);
			if (edge0.v0.x==edge0.v1.x || edge0.v0.y==edge0.v1.y )
				continue;
			for(int j=i+1; j<edgeVisuals.size(); j++){
				EdgeVisual edge1 = edgeVisuals.get(j);
				if (edge1.v0.x==edge1.v1.x || edge1.v0.y==edge1.v1.y )
					continue;
				Vector2D vec0 = new Vector2D(edge0.v0, edge0.v1);
				Vector2D vec1 = new Vector2D(edge1.v0, edge1.v1);
				vec0.normalize();
				vec1.normalize();
				if (Math.abs(Vector2D.cross_product(vec0, vec1))<0.1){
					String sign = (Vector2D.dot_product(vec0, vec1)>0) ? "" : "Pi + ";
					String equation = ""+edge0.get_variable("angle")+ " = " + sign + edge1.get_variable("angle");
					angle_equations.add(equation);
					
					if (edge0.get_variable("angle").startsWith("angle"))
						change_constant_property_free(edge0, "angle");
					if (edge1.get_variable("angle").startsWith("angle"))
						change_constant_property_free(edge1, "angle");

				}
			}
		}
		
		
	}
	public static void change_constant_property_free(Visual visual, String property_name){
		Property property = visual.get_property(property_name);
		property.constant = false;
	}
	public static EdgeVisual pick_edge_visual(Vector2D v){
		double min = Tool.d/2;//Double.MAX_VALUE;
		EdgeVisual closest = null;
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof EdgeVisual){
				double d = ((EdgeVisual)visual).get_distance(v);
				if (d < min){
					min = d;
					closest = (EdgeVisual)visual;
					
				}
			}
		}
		return closest;
		
	}
	
	
	public static void set_angle(Arc arc, EdgeVisual visual1, Vector2D vec0, Vector2D vec1, String label){

		String offset = "0";
		
		if (is_number(label))
			label = convert_to_radian(label);
		
		if (Vector2D.distance(arc.center,  visual1.v1) < Vector2D.distance(arc.center,  visual1.v0)){
			offset = "Pi";
		}

		if (vec0.y == 0){
			if (vec0.x < 0)
				label = "( Pi - "+label+ " )";
			if (vec1.y < 0)
				label = "( 2*Pi - "+label+ " )";
			visual1.set_property("angle",  "( "+label+" + "+offset+")", true, false);
		}
		else if (vec0.x == 0){
			if (vec0.y > 0)
				if (vec1.x > 0)
					label = "( Pi/2 - "+label+ " )";
				else
					label = "( Pi/2 + "+label+ " )";
			else
				if (vec1.x > 0)
					label = "( -Pi/2 + "+label+ " )";
				else
					label = "( - Pi/2 - "+label+ " )";
				
		
			visual1.set_property("angle",  "( "+label+" + "+offset+")", true, false);
		}
	}
	public static void set_angle(Arc arc, EdgeVisual visual0, EdgeVisual visual1, Vector2D vec0, Vector2D vec1, String label){
		if (is_number(label))
			label = convert_to_radian(label);

		String angle0 = get_angle(visual0, arc.center);
		String angle1 = get_angle(visual1, arc.center);

		double angle0_onscreen = Vector2D.get_angle_360(new Vector2D(1,0), vec0);
		double angle1_onscreen = Vector2D.get_angle_360(new Vector2D(1,0), vec1);
		
		String equation = (angle1_onscreen > angle0_onscreen) ?
				angle1 + " - " + angle0 + " = " + label
				:
				angle0 + " - " + angle1 + " = " + label;
		
		angle_equations.add(equation);
	}
	static public ArrayList<String> angle_equations = new ArrayList();
	
	static String get_angle(EdgeVisual visual, Vector2D center){
		if (Vector2D.distance(center,  visual.v1) < Vector2D.distance(center,  visual.v0)){
			return " ( "+ visual.get_variable("angle") + " + Pi )";
		}
		else
			return visual.get_variable("angle");
	}
	
	
	static boolean is_number(String label){
		try {
			Integer.parseInt(label);
			return true;
		} catch (NumberFormatException nfex) {
			return false;
		}
	}
	static String convert_to_radian(String label){
		int angle = Integer.parseInt(label);
		return " Pi * " + label + " / 180 ";
	}
	/*
	public static void set_angle(Arc arc, EdgeVisual visual1, Vector2D vec0, Vector2D vec1, String label){

		int sign_x = 1;
		int sign_y = 1;
		
		if (Vector2D.distance(arc.center,  visual1.v1) < Vector2D.distance(arc.center,  visual1.v0)){
			sign_x *= -1;
			sign_y *= -1;
		}

		if (vec0.y == 0){
			if (vec0.x < 0)
				sign_x *= -1;
			
			//visual1.angle_x = ""+sign_x+"*"+ "cos("+label+")";
			//visual1.angle_y = ""+sign_y+"*"+ "sin("+label+")";
			
			visual1.set_property("angle",  label);
		}
		else if (vec0.x == 0){
			if (vec0.y < 0)
				sign_y *= -1;
		
			visual1.angle_x = ""+sign_x+"*"+ "sin("+label+")";
			visual1.angle_y = ""+sign_y+"*"+ "cos("+label+")";
		}
	}
	*/
				
	static Bucket get_nearest_bucket(Vector2D v){
		ArrayList<Bucket> buckets = MainPanel.mainPanel.buckets;
		Bucket nearest = null;
		double min = Double.MAX_VALUE;
		
		for(Bucket bucket: buckets){
			double d = Vector2D.distance(bucket.position,  v);
			if (d < min){
				min = d;
				nearest = bucket;
			}
		}
		return nearest;
	}
	static Connection get_nearest_connection(Vector2D v){
		ArrayList<Connection> connections = MainPanel.mainPanel.connections;
		Connection nearest = null;
		double min = Double.MAX_VALUE;
		
		for(Connection connection : connections){
			double d = Vector2D.distance(connection.position,  v);
			if (d < min){
				min = d;
				nearest = connection;
			}
		}
		return nearest;
	}
	static void add_gap_label(Node node0, Node node1, String label){
		//label = insert_multiplier(label);
		
		HashMap<Node, String> node_to_string = MainPanel.mainPanel.gap_labels.get(node0);
		if (node_to_string == null){
			node_to_string = new HashMap<Node, String>();
			 MainPanel.mainPanel.gap_labels.put(node0, node_to_string);
		}
		node_to_string.put(node1,  label);
	}
	
	static String remove_numeric_part(String label){
		// 0.1L -> L
		int index = get_index_of_alphabet(label);
		
		if (index == 0)
			return label;
		if (index == -1)
			return null;
		
		return label.substring(index, label.length());
		
	}
	static String insert_multiplier(String label){
		// "0.1L -> 0.1*L
		
		int index = get_index_of_alphabet(label);
		
		if (index == 0 || index == -1)
			return label;
		
		return label.substring(0,index)+"*"+label.substring(index, label.length());
	}
	static int get_index_of_alphabet(String label){
		char[] nums = {'0','1','2','3','4','5','6','7','8','9','.'};
		
		for(int i=0; i<label.length(); i++){
			if (contains(nums, label.charAt(i)))
					continue;
			return i;
		}
		return -1;
	}
	static boolean contains(char[] nums, char c){
		for(int i=0; i<nums.length; i++)
			if (nums[i] == c)
				return true;
		return false;
	}
	static String get_gap_label(Node node0, Node node1){
		HashMap<Node, String> node_to_string = MainPanel.mainPanel.gap_labels.get(node0);
		if (node_to_string == null){
			return null;
		}
		return node_to_string.get(node1);
	}
	
	
	public static void analyze_tags(){
		ArrayList<Visual> visuals = MainPanel.mainPanel.visuals;
		for(Visual visual : visuals){
			visual.tags.clear();
		}
		for(Visual visual : visuals){
			if (visual instanceof Tag){
				String label = ((Tag)visual).label;
				Vector2D v = ((Tag)visual).v0;
				String[] labels = label.split(" ",0);
				
				Visual closest_visual = find_closest_nontag_visual(v, visuals);
				if (closest_visual == null)
					continue;
				for(String l : labels)
					closest_visual.tags.add(l);
			}
		}

		
		/*
		ArrayList<Visual> visuals = MainPanel.mainPanel.visuals;
		for(Visual visual : visuals){
			visual.label = visual.get_default_name();
		}
		for(Visual visual : visuals){
			if (visual instanceof Tag){
				String label = ((Tag)visual).label;
				Vector2D v = ((Tag)visual).v0;
				
				Visual closest_visual = find_closest_nontag_visual(v, visuals);
				closest_visual.label = label;
			}
		}
		*/
		
	}
	
	public static Visual find_closest_nontag_visual(Vector2D v, ArrayList<Visual> visuals){
		Visual closest = null;
		double min = Double.MAX_VALUE;
		for(Visual visual : visuals){
			if (visual instanceof Tag || visual instanceof Gap || visual instanceof Arc)
				continue;
			
			double d = visual.get_distance(v);
			if (d < min){
				min = d;
				closest = visual;
			}
		}
		return closest;
	}
	public static void analyze_friction(){
		for(Mass mass : MainPanel.mainPanel.masses()){
			mass.friction = null;
			for(Wall wall : MainPanel.mainPanel.walls()){
				if (wall.get_distance(mass.v0) < mass.r+5){
					
					
					String normal = wall.get_property_value("angle");
					Vector2D vec = new Vector2D(wall.v0, mass.v0);
					if (Vector2D.dot_product(new Vector2D(wall.v0, wall.v1), vec)  > 0)
						normal = "("+normal+"+Pi/2)";
					else
						normal = "("+normal+"-Pi/2)";

					
					String tangent = "";
									
					if (mass.direction != null){
						
						if (mass.direction.equals("right"))
							tangent = "Pi";
						else if (mass.direction.equals("left"))
							tangent = "0";
						else if (mass.direction.equals("up"))
							tangent = "Pi/2";
						else if (mass.direction.equals("down"))
							tangent = "3*Pi/2";
						else {
							int sign = 1;
							if (mass.direction.equals("downward") || mass.direction.equals("down_slope"))
								sign *= -1;
							if (wall.v0.y > wall.v1.y)
								sign *= -1;
							if (sign > 0)
								tangent = wall.get_property_value("angle");
							else
								tangent = "("+wall.get_property_value("angle")+"+Pi)";
						}
						
						mass.friction = new Friction(normal, tangent);
						continue;
						
					}
					
					
					
					
					
					Vector2D pull_direction = null;
					
					// 2012-Jun-3-4
					EdgeVisual edgeVisual = get_connected_arrow(mass);						
					
					if (edgeVisual == null)
						edgeVisual = get_connected_edgeVisual(mass);
					
					if (edgeVisual == null){
						if (wall.v0.y > wall.v1.y)
							tangent = "( "+ normal + " - Pi/2 )";
						else
							tangent = "( "+ normal + " + Pi/2 )";
						mass.friction = new Friction(normal, tangent);
						continue;
					}
					
					Vector2D v1 = edgeVisual.get_opposite_vertex(mass.v0);
					pull_direction = new Vector2D(mass.v0, v1);
					
					
					
					Vector2D wall_direction = new Vector2D(wall.v0, wall.v1);
					if ( Vector2D.dot_product(pull_direction,  wall_direction) == 0){
						// pull is perpendicular to wall
						Vector2D gravity_direction = new Vector2D(0,1);
						if ( Vector2D.dot_product(gravity_direction,  wall_direction) > 0)
							tangent = "( "+ normal + " + Pi/2 )";
						else
							tangent = "( "+ normal + " - Pi/2 )";
					}
					else {
						// pull is parallel to wall
						
						if ( Vector2D.dot_product(pull_direction,  wall_direction) > 0)
							tangent = "( "+ normal + " + Pi/2 )";
						else
							tangent = "( "+ normal + " - Pi/2 )";
					}
					mass.friction = new Friction(normal, tangent);
				}
			}
		}
		
		
	}

	/*
	public static void analyze_friction(){
		for(Mass mass : MainPanel.mainPanel.masses()){
			mass.friction = null;
			for(Wall wall : MainPanel.mainPanel.walls()){
				if (wall.get_distance(mass.v0) < mass.r+5){
					
					
					String normal = wall.get_property_value("angle");
					Vector2D vec = new Vector2D(wall.v0, mass.v0);
					if (Vector2D.dot_product(new Vector2D(wall.v0, wall.v1), vec)  > 0)
						normal = "("+normal+"+Pi/2)";
					else
						normal = "("+normal+"-Pi/2)";
						
					String tangent = "";
					
					Vector2D pull_direction = null;
					
					// 2012-Jun-3-4
					EdgeVisual edgeVisual = get_connected_arrow(mass);						
					
					if (edgeVisual == null)
						edgeVisual = get_connected_edgeVisual(mass);
					
					if (edgeVisual == null){
						if (wall.v0.y > wall.v1.y)
							tangent = "( "+ normal + " - Pi/2 )";
						else
							tangent = "( "+ normal + " + Pi/2 )";
						mass.friction = new Friction(normal, tangent);
						return;
					}
					
					Vector2D v1 = edgeVisual.get_opposite_vertex(mass.v0);
					pull_direction = new Vector2D(mass.v0, v1);
					
					
					
					Vector2D wall_direction = new Vector2D(wall.v0, wall.v1);
					if ( Vector2D.dot_product(pull_direction,  wall_direction) == 0){
						// pull is perpendicular to wall
						Vector2D gravity_direction = new Vector2D(0,1);
						if ( Vector2D.dot_product(gravity_direction,  wall_direction) > 0)
							tangent = "( "+ normal + " + Pi/2 )";
						else
							tangent = "( "+ normal + " - Pi/2 )";
					}
					else {
						// pull is parallel to wall
						
						if ( Vector2D.dot_product(pull_direction,  wall_direction) > 0)
							tangent = "( "+ normal + " + Pi/2 )";
						else
							tangent = "( "+ normal + " - Pi/2 )";
					}
					mass.friction = new Friction(normal, tangent);
				}
			}
		}
		
		
	}
*/
	public static EdgeVisual get_connected_arrow(Mass mass){
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof Arrow){
				Arrow wire = (Arrow) visual;
				if (Vector2D.distance(wire.v0, mass.v0) == 0)
					return wire;
				if (Vector2D.distance(wire.v1, mass.v0) == 0)
					return wire;
			}
		}
		return null;
	}
	public static EdgeVisual get_connected_wire(Mass mass){
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof Wire){
				Wire wire = (Wire) visual;
				if (Vector2D.distance(wire.v0, mass.v0) == 0)
					return wire;
				if (Vector2D.distance(wire.v1, mass.v0) == 0)
					return wire;
			}
		}
		return null;
	}
	public static ArrayList<Arrow> get_connected_arrows(Mass mass){
		ArrayList<Arrow> arrows = new ArrayList();
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof Arrow){
				Arrow wire = (Arrow) visual;
				if (Vector2D.distance(wire.v0, mass.v0) == 0)
					arrows.add(wire);
				if (Vector2D.distance(wire.v1, mass.v0) == 0)
					arrows.add(wire);
			}
		}
		return arrows;
	}
	public static ArrayList<Wire> get_connected_wires(Mass mass){
		ArrayList<Wire> wires = new ArrayList();
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof Wire){
				Wire wire = (Wire) visual;
				if (Vector2D.distance(wire.v0, mass.v0) == 0)
					wires.add(wire);
				if (Vector2D.distance(wire.v1, mass.v0) == 0)
					wires.add(wire);
			}
		}
		return wires;
	}

	static EdgeVisual get_connected_edgeVisual(Mass mass){
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof EdgeVisual){
				EdgeVisual wire = (EdgeVisual) visual;
				if (Vector2D.distance(wire.v0, mass.v0) == 0)
					return wire;
				if (Vector2D.distance(wire.v1, mass.v0) == 0)
					return wire;
			}
		}
		return null;
	}
	public static void analyze_connectivity(){
		ArrayList<Connection> connections = MainPanel.mainPanel.connections;
		connections.clear();
	
		ArrayList<Bucket> buckets = MainPanel.mainPanel.buckets;
		buckets.clear();
		
		ArrayList<Visual> visuals = MainPanel.mainPanel.visuals;
		for(Visual visual : visuals){
			if (visual instanceof Stick)
				((Stick)visual).clear_nodes();
			if (visual instanceof Wall)
				((Wall)visual).clear_nodes();
		}
		
		String output = "";
		
		

		
		//
		// end points
		//
		for(Visual visual : visuals){
			ArrayList<Node> nodes = visual.get_nodes();
			for(Node node : nodes){
				Bucket bucket = get_bucket(buckets, node.position);
				if (bucket == null){
					bucket = new Bucket(node);
					buckets.add(bucket);
				}
				else {
					bucket.add(node);
				}
			}
		}
		
		
		
		//
		// middle points
		//
		for(Visual visual : visuals){
			
			if (visual instanceof Wall){
				Edge2D edge = new Edge2D(((Wall)visual).v0, ((Wall)visual).v1);
				for(Bucket bucket : buckets){
					if (edge.distance_as_a_segment(bucket.position) == 0){
						bucket.add(((Wall)visual).get_node(bucket.position));
					}
				}
			}
			
			else if (visual instanceof Stick){
				Edge2D edge = new Edge2D(((Stick)visual).v0, ((Stick)visual).v1);
				System.out.println("bar "+edge.start+" "+edge.end);
				for(Bucket bucket : buckets){
					System.out.println("bucket "+bucket.position);
					if (Vector2D.distance(edge.start, bucket.position)==0)
						continue;
					if (Vector2D.distance(edge.end, bucket.position)==0)
						continue;
					if (edge.distance_as_a_segment(bucket.position) == 0){
						System.out.println("  get_node");
						bucket.add(((Stick)visual).get_node(bucket.position));
					}
				}
			}
			
		}		
		
		
		for(Bucket bucket : buckets){
			//if (bucket.nodes.size()>1)
				connections.add(new Connection(bucket.position, bucket.nodes));
		}
		//MainPanel.mainPanel.textCanvas.println(output);
	}
	static Bucket get_bucket(ArrayList<Bucket> buckets, Vector2D position){
		for(Bucket bucket : buckets){
			if (Vector2D.distance(bucket.position,  position) < Tool.d / 2)
				return bucket;
		}
		return null;
	}
	
	/*
	public static void analyze_connectivity(){
		ArrayList<Connection> connections = MainPanel.mainPanel.connections;
		connections.clear();
		
		ArrayList<Visual> visuals = MainPanel.mainPanel.visuals;
		String output = "analyze\n";
		for(int i=0; i<visuals.size()-1;  i++){
			Visual visual0 = visuals.get(i);
			for(int j=i+1; j<visuals.size();  j++){
				Visual visual1 = visuals.get(j);
				
				ArrayList<Node> nodes0 = visual0.get_nodes();
				ArrayList<Node> nodes1 = visual1.get_nodes();
				
				for(Node node0 : nodes0){
					for(Node node1 : nodes1){
						if (Vector2D.distance(node0.position, node1.position) == 0){
							output+=("connect("+node0.label+","+node1.label+")\n");
							connections.add(new Connection(node0, node1));
						}
					}
				}
			}			
		}
		MainPanel.mainPanel.textCanvas.println(output);
	}
	*/
}

class Bucket{
	public ArrayList<Node> nodes = new ArrayList();
	Vector2D position;
	Bucket(Node node){
		position = node.position;
		nodes.add(node);
	}
	void add(Node node){
		nodes.add(node);
	}
	
}