package physics;

import java.awt.*;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.util.*;

import sun.security.tools.keytool.Main;

public class Arc extends Visual {
	public Vector2D v0;
	public Vector2D v1;
	public Vector2D center;
	//String label = "";
	public Arc(){
		super();

	}
	public Arc(Vector2D v0, Vector2D v1){
		super();

		this.v0 = v0;
		this.v1 = v1;
	}
	public void save(BufferedWriter out){
		Record.write(out, "v0", v0);
		Record.write(out, "v1", v1);
		Record.write(out, "center", center);
		Record.write(out,  "label", label);
	}
	public void load(BufferedReader in){
		v0 = Record.read_Vector2D(in);
		v1 = Record.read_Vector2D(in);
		center = Record.read_Vector2D(in);
		label = Record.read_String(in);
	}
	
	
	public String get_default_name(){
		return "gap"+id;
	}

	public void paint_property(Graphics g, Point offset){
		
		println(g,"Gap"+id, offset);
		println(g,"Length: L", offset);
		
	}
	
	
	
	
	int drag_status = 0;
	static final int DRAG_START = 0;
	static final int DRAG_END = 1;
	static final int DRAG_BODY = 2;
	static final int DRAG_CENTER = 3;
	
	Vector2D prev_p = null;
	public void start_dragging(Vector2D p){
		prev_p = p;
		
		if (Vector2D.distance(p,  v0)<Def.pick_radius)
			drag_status = DRAG_START;
		else if (Vector2D.distance(p,  v1)<Def.pick_radius)
			drag_status = DRAG_END;
		else if (Vector2D.distance(p,  center)<Def.pick_radius)
			drag_status = DRAG_CENTER;
		else
			drag_status = DRAG_BODY;
	}
	public void dragging(Vector2D p){
		Vector2D vec = new Vector2D(prev_p, p);
		switch(drag_status){
		case DRAG_START:
			v0.add(vec);
			break;
		case DRAG_END:
			v1.add(vec);
			break;
		case DRAG_CENTER:
			center.add(vec);
			break;
		case DRAG_BODY:
			v0.add(vec);
			v1.add(vec);
			center.add(vec);
			break;
		}
		
		prev_p = p;
	}
	public void finish_dragging(){
		switch(drag_status){
		case DRAG_START:
			v0 = snap_to_edge(v0);
			break;
		case DRAG_END:
			v1 = snap_to_edge(v1);
			break;
		case DRAG_CENTER:
			center = Tool.snap(center);
			break;
		case DRAG_BODY:
			v0 = snap_to_edge(v0);
			v1 = snap_to_edge(v1);
			center = Tool.snap(center);
			break;
		}
	}
	public Vector2D snap_to_edge(Vector2D v){
		double min = Tool.d;
		Vector2D closest = null;
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof EdgeVisual){
				EdgeVisual edge = (EdgeVisual) visual;
				double d = edge.get_distance(v);
				if (d < min){
					min = d;
					closest = edge.get_nearest_point(v);
				}
			}
		}
		if (closest != null)
			return closest;
		
		return Tool.snap(v);
	}

	
	
	
	
	
	
	
	
	double d = 10;
	ArrayList<Vector2D>  points;
	public void paint(Graphics g){
		if (Vector2D.distance(v0, v1)==0)
			return;
		
		
		
		
		double x0 = Double.MAX_VALUE;
		double y0 = Double.MAX_VALUE;
		double x1 = -Double.MAX_VALUE;
		double y1 = -Double.MAX_VALUE;
		
		
		
		Vector2D vec0 = new Vector2D(center, v0);
		Vector2D vec1 = new Vector2D(center, v1);
		double r0 = vec0.length();
		double r1 = vec1.length();
		vec0.normalize();
		double angle = Vector2D.get_angle_360(vec0,  vec1);
		if (angle >180){
			angle = angle-360;
		}
		points = new ArrayList();
		int n = (int)(Math.abs(angle) / 10);
		for(int i=0; i<=n; i++){
			double a = angle * i / n;
			Vector2D vec = Vector2D.rotate(vec0,  angle * i / n);
			vec.multiply( r1 * i / n + r0 * (1- 1.0*i/n));
			Vector2D p = Vector2D.add(center, vec); 
			points.add(p);
			
			x0 = Math.min(x0,  p.x);
			y0 = Math.min(y0,  p.y);
			x1 = Math.max(x1,  p.x);
			y1 = Math.max(y1,  p.y);
		}
		bbox = new Rectangle((int)x0, (int)y0, (int)(x1-x0), (int)(y1-y0));
		
		
		g.setColor(Color.black);
		for(int i=0; i< points.size()-1; i++){
			Vector2D p0 = (Vector2D)points.get(i);
			Vector2D p1 = (Vector2D)points.get(i+1);
			if(p0!=null && p1!=null){
				g.drawLine((int)p0.x, (int)p0.y, (int)p1.x, (int)p1.y);
			}
		}	
		
		
		
		
		Vector2D mid_point = Vector2D.add(center,  Vector2D.rotate(new Vector2D(center, v0),  angle/2));
		Vector2D vec = new Vector2D(center, mid_point);
		vec.normalize();
		mid_point = Vector2D.add(mid_point, Vector2D.multiply(vec,  15));

		//g.drawRect((int)mid_point.x-5,  (int)mid_point.y-5, 10,10);  
		
		
		//g.drawString(label, (int)mid_point.x-10, (int)mid_point.y+10);
		drawString(g, label,  (int)mid_point.x-10, (int)mid_point.y+10);
	}
	Rectangle bbox;
	public void paint_bbox(Graphics g){
		g.setColor(Color.red);
	
		//g.drawRect(bbox.x, bbox.y, bbox.width, bbox.height);

	
	
		
		int r = 10;
		g.drawOval((int)(center.x-r),  (int)(center.y-r), r*2, r*2);

		g.drawOval((int)(v0.x-r),  (int)(v0.y-r), r*2, r*2);
		g.drawOval((int)(v1.x-r),  (int)(v1.y-r), r*2, r*2);
}
	
	public double get_distance_endpoint(Vector2D v){
		double min =  Math.min(Vector2D.distance(v0, v), Vector2D.distance(v1, v));
		if (MainPanel.mainPanel.selected_visual == this)
			return Math.min(Vector2D.distance(center,  v), min);
		else
			return min;
	}
	public double get_distance(Vector2D v){
		double stroke_distance = Stroke.distance(points, v);
		
	
		if (MainPanel.mainPanel.selected_visual != this)
			return stroke_distance;
		
		double center_distance = Vector2D.distance(center,  v);
		
		
		return Math.min(stroke_distance, center_distance);
		//return (new Edge2D(v0, v1).distance_as_a_segment(v));
	}
	
	public void set_label(String typed){
		int code = (int)typed.charAt(0);
		if (code == 8 || code == 127){
			if (label.length()>0)
				label = label.substring(0, label.length()-1);
		}
		else
			label += typed;
	}
	public ArrayList<Node> get_nodes(){
		return new ArrayList();
	}
}
