package physics;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.*;

public class EdgeVisual extends Visual {
	public Vector2D v0;
	public Vector2D v1;
	
	//String angle = null;
	public EdgeVisual(){
		super();
	}
	public void save(BufferedWriter out){
		Record.write(out, "v0", v0);
		Record.write(out, "v1", v1);
	}
	public void load(BufferedReader in){
		v0 = Record.read_Vector2D(in);
		v1 = Record.read_Vector2D(in);
	}
	
	
	
	
	public EdgeVisual(Vector2D v0, Vector2D v1){
		super();

		this.v0 = v0;
		this.v1 = v1;
		
		set_property("angle",  "angle"+id, true, false);
	}
	
	static public void paint_arrowhead(Graphics g, Vector2D v0, Vector2D v1, double d){
		Vector2D x_vec = new Vector2D(v0, v1);
		x_vec.normalize();
		x_vec.multiply(d);
		Vector2D y_vec = Vector2D.rotate90(x_vec);
		drawLine(g, v0, Vector2D.add(v0, x_vec, y_vec));
		drawLine(g, v0, Vector2D.add(v0, x_vec, Vector2D.multiply(y_vec,-1)));
	}
	
	public Vector2D get_opposite_vertex(Vector2D v){
		if (Vector2D.distance(v,  v0) < Vector2D.distance(v, v1)){
			return v1;
		}
		else
			return v0;
		
	}
	public void set_default_angle(){
		if (Math.abs(v0.y -v1.y)<Tool.d){
			if (v1.x > v0.x)
				set_property("angle", "0", true, false);	// is_constant,  not always positive
			else
				set_property("angle", "Pi", true, false);
		}
		else if (Math.abs(v0.x -v1.x)<Tool.d){
			if (v1.y > v0.y)
				set_property("angle", "Pi/2", true, false);
			else
				set_property("angle", "Pi*3/2", true, false);
		}
		else
			set_property("angle",  "angle"+id, true, false);
		
	}
	public String vec_x(Vector2D v){
		if (v0 == v)
			return vec_x();
		else
			return negate(vec_x());
	}
	public String vec_y(Vector2D v){
		if (v0 == v)
			return vec_y();
		else
			return negate(vec_y());
	}
	
	public String vec_x(){
		return "cos("+get_variable("angle")+")";
	}
	public String vec_y(){
		return "sin("+get_variable("angle")+")";
	}
	
	
	
	
	int drag_status = 0;
	static final int DRAG_START = 0;
	static final int DRAG_END = 1;
	static final int DRAG_BODY = 2;
	
	Vector2D prev_p = null;
	public void start_dragging(Vector2D p){
		prev_p = p;
		
		if (Vector2D.distance(p,  v0)<Def.pick_radius)
			drag_status = DRAG_START;
		else if (Vector2D.distance(p,  v1)<Def.pick_radius)
			drag_status = DRAG_END;
		else
			drag_status = DRAG_BODY;
	}
	public void dragging(Vector2D p){
		Vector2D vec = new Vector2D(prev_p, p);
		switch(drag_status){
		case DRAG_START:
			v0.add(vec);
			break;
		case DRAG_END:
			v1.add(vec);
			break;
		case DRAG_BODY:
			v0.add(vec);
			v1.add(vec);
			break;
		}
		
		prev_p = p;
	}
	public void finish_dragging(){
//			v0 = Tool.snap(v0);
//			v1 = Tool.snap(v1);
		switch(drag_status){
		case DRAG_START:
			//v0 = Tool.snap(v0);
			v0 = snap_to_canonical_angles(v1, v0);
			break;
		case DRAG_END:
			//v1 = Tool.snap(v1);
			v1 = snap_to_canonical_angles(v0, v1);
			break;
		case DRAG_BODY:
			v0 = Tool.snap(v0);
			v1 = Tool.snap(v1);
			break;
		}

	}
	static Vector2D snap_to_canonical_angles(Vector2D v0, Vector2D v1){
		// ԋ߂ canonical Ȃ̂T
		int index = 0;
		
		ArrayList<Vector2D> candidates = get_candidates(v1);
		
		for(int i=0; i<candidates.size(); i++){
			Vector2D v = candidates.get(i);
			if (is_canonical(v0, v))
				return v;
		}
		return v1;
		/*
		double dx = (v1.x-v0.x);
		double dy = (v1.y-v0.y);
		int nx = (int)Math.floor(0.5+dx/Tool.d);
		int ny = (int)Math.floor(0.5+dy/Tool.d);
		
		int abs_nx = Math.abs(nx);
		int abs_ny = Math.abs(ny);
		
		if (abs_nx == 0 || abs_ny == 0 || abs_nx == abs_ny)
			return v1;
		else if (abs_nx > abs_ny){
			nx = nx * abs_ny * 2 / abs_nx;
			return new Vector2D(v0.x + Tool.d * nx, v1.y);
			
		}
		else if (abs_nx < abs_ny){
			ny = ny * abs_nx * 2 / abs_ny;
			return new Vector2D(v1.x, v0.y + Tool.d * ny);
		}
		else
			return v1;
		*/
	}
	static boolean is_canonical(Vector2D v0, Vector2D v1){
		double dx = (v1.x-v0.x);
		double dy = (v1.y-v0.y);
		int nx = (int)Math.floor(0.5+dx/Tool.d);
		int ny = (int)Math.floor(0.5+dy/Tool.d);
		
		int abs_nx = Math.abs(nx);
		int abs_ny = Math.abs(ny);
		
		if (abs_nx == 0 || abs_ny == 0) // || abs_nx == abs_ny)
			return true;
		if (abs_nx > abs_ny)
			return abs_nx == abs_ny * 2;
		if (abs_nx < abs_ny)
			return abs_ny == abs_nx * 2;
		return false;
		
	}
	static ArrayList<Vector2D> get_candidates(Vector2D v){
		ArrayList<Vector2D> candidates = new ArrayList<Vector2D>();
		int x0 = (int) Math.floor(v.x / Tool.d);
		int y0 = (int) Math.floor(v.y / Tool.d);
		
		for(int x=-3; x<=3; x++)
			for(int y=-3; y<=3; y++)
				candidates.add(new Vector2D( (x0+x)*Tool.d, (y0+y)*Tool.d));
		
		Collections.sort(candidates, new SampleComparator(v));
		
		return candidates;
	}

	

	public double get_distance_endpoint(Vector2D v){
		return Math.min(Vector2D.distance(v0, v), Vector2D.distance(v1, v));
	}
	public double get_distance(Vector2D v){
		return (new Edge2D(v0, v1).distance_as_a_segment(v));
	}
	public Vector2D get_nearest_point(Vector2D v){
		return (new Edge2D(v0, v1).get_nearest_point(v));
	}
	
	

	
}
class SampleComparator implements Comparator<Vector2D>{
	Vector2D center;
	SampleComparator(Vector2D _center){
		center = _center;
	}
	public int compare(Vector2D a, Vector2D b) {
		return (int)(Vector2D.distance(center,  a) - Vector2D.distance(center,  b));
	}
}
