package physics;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.geom.*;

public class Finger extends Visual{

	
	public Vector2D v0;
	public double angle = 0;
	String label = "";
	public Finger(){
		super();

	}
	
	public Finger(Vector2D v0){
		super();

		this.v0 = v0;
		
		//set_property("mass", "m"+id, true);
		//set_property("force", "f"+id, false);
	}
	
	public void save(BufferedWriter out){
		Record.write(out, "v0", v0);
		Record.write(out, "angle", angle);
	}
	public void load(BufferedReader in){
		v0 = Record.read_Vector2D(in);
		angle = Record.read_double(in);
	}
	
	
	
	public String get_default_name(){
		return "finger"+id;
	}

	
	int drag_target = 0;
	static final int BODY = 0;
	static final int HANDLE = 1;
	Vector2D prev_p = null;
	Vector2D grabbed_handle = null;
	public void start_dragging(Vector2D p){
		prev_p = p;
		if (Vector2D.distance(handle(),  p)<r){
			drag_target = HANDLE;
			grabbed_handle = handle();
		}
		else
			drag_target = BODY;
	}
	public void dragging(Vector2D p){
		Vector2D vec = new Vector2D(prev_p, p);
		switch(drag_target){
			case BODY:
				v0.add(vec);
				break;
			case HANDLE:
				grabbed_handle.add(vec);
				Vector2D vec0 = new Vector2D(-1,0);
				angle = Vector2D.get_angle_360(vec0,  new Vector2D(v0, grabbed_handle));
				break;
		}
		prev_p = p;
	}
	
	public void finish_dragging(){
		v0 = Tool.snap(v0);
		
}
	
	
	public void paint(Graphics g){
		g.setColor(Color.black);
		
		//g.translate((int)v0.x, (int)v0.y);
		
	    Graphics2D g2 = (Graphics2D) g;
	    AffineTransform af = new AffineTransform();
	    AffineTransform af2 = g2.getTransform();
		af.translate(v0.x, v0.y);
		af.rotate(Math.PI*angle/180);
	    g2.setTransform(af);
	    
		for(Stroke stroke : strokes){
			stroke.paint(g);
		}
		
		
		// g.translate(-(int)v0.x, -(int)v0.y);
	    g2.setTransform(af2);
		
	}
	double r = 10;
	//Vector2D handle=null;
	public Vector2D handle(){
		Vector2D vec = new Vector2D(-70,0);
		vec = Vector2D.rotate(vec, angle);
		return Vector2D.add(v0,  vec);
	}
	public void paint_bbox(Graphics g){
		g.setColor(Color.red);
		g.drawOval((int)(v0.x-r),  (int)(v0.y-r),  (int)(2*r), (int)(2*r));
		
		g.drawOval((int)(handle().x-r),  (int)(handle().y-r),  (int)(2*r), (int)(2*r));
		
	}
	public double get_distance(Vector2D v){
		if (MainPanel.mainPanel.selected_visual == this)
			return Math.min(Vector2D.distance(v0,  v), Vector2D.distance(handle(),  v));
		else
			return Vector2D.distance(v0,  v);
	}
	public void set_label(String label){
		this.label = label;
	}
	public ArrayList<Node> get_nodes(){
		
		Node node =  new Node(this,  "reaction", "reaction", v0, "id"+id);
		
		ArrayList<Node> nodes = new ArrayList();
		nodes.add(node);
		return nodes;
	}
	
	
	

	
	
	
	
	
	
	

	
	
	
	
	
	static ArrayList<Stroke> strokes;
	
	static {
		strokes = load_strokes("finger.txt");
		
		for(Stroke stroke : strokes){
			for(Vector2D v : stroke.points){
				v.x -= 23;
				v.y += 60;
				
				v.x *= 0.1;
				v.y *= 0.1;
				
			}
		}
	}
	
	
	static public ArrayList<Stroke> load_strokes(String filename){
		ArrayList<Stroke> strokes = null;
		try{
		BufferedReader in = new BufferedReader(new FileReader(filename));
		strokes = load_strokes(in);
		in.close();
		}
		catch(IOException e) { e.printStackTrace();}
		return strokes;
	}
	
	static public ArrayList<Stroke> load_strokes(BufferedReader in) throws IOException{
		ArrayList<Stroke> strokes = new ArrayList();
		
		while(true){
			String line = in.readLine();	
			if (line == null)
				break;
			if (line.startsWith("<stroke>")){
				Stroke stroke = new Stroke();
				stroke.load(in);
				strokes.add(stroke);
			}
		
		}
		return strokes;
	}
	
}
