package physics;

import java.util.*;

public class LF {
	static HashMap<String, Assignment> assignments = new HashMap();
	static HashMap<String, Property> vars = new HashMap();

	static public HashSet<String> bounded_variables = new HashSet();
	
	static public void process_lines(String text) {
		bounded_variables.clear();
		
		String[] lines = text.split("\n");
		for (String line : lines) {
			process_line(line);
		}

	}

	static public void process_line(String line) {
		println(line);
		
		String[] words = line.split("[(),]", 0);
		String a = "";
		for(String word : words)
			a += " | "+word;
		//println(a);
		
		String[] entity_names = new String[] { "spring", "stick", "mass", "line", "hand","floor","pulley","wall","fixpoint" };
		String[] property_names = new String[] { "natural_length", "cnst_spring", "growth", "length", "weight",
				"elastic_energy",  "force" };
		String[] property_names_1 = new String[] { "tensforce"};
		String[] property_names_2 = new String[] { "sfc", "friction_force", "rel_force"};///, "angle"

		//System.out.println(""+line);
		if (is_one_of(words[0], entity_names)) {
			String label = words[1];
			assignments.put(label, new Assignment(words[0], label));
			println("set_assignment " + words[0] + " " + label);
		}

		else if (is_one_of(words[0], property_names)) {//wall(w)
			String label = words[1];
			String var = words[2];
			Assignment assignment = assignments.get(label);
			Property property = new Property(words[0]);
			assignment.properties.add(property);
			vars.put(var, property);

			println("set_property " + label + " " + words[0] + " " + var);
		}
		else if (is_one_of(words[0], property_names_1)) {//tensforce(l,ep2(l),var2)
			String label = words[1];
			String var = words[5];
			Assignment assignment = assignments.get(label);
			Property property = new Property(words[0]);
			assignment.properties.add(property);
			vars.put(var, property);

			println("set_property " + label + " " + words[0] + " " + var);
		}
		else if (is_one_of(words[0], property_names_2)) { // sfc(B,id3,var5)
			String label0 = words[1];
			String label1 = words[2];
			String var = words[3];
			{
			Assignment assignment0 = assignments.get(label0);
			Assignment assignment1 = assignments.get(label1);
			Property property = new Property(words[0]);
			assignment0.properties.add(property);
			assignment1.properties.add(property);
			vars.put(var, property);
			println("set_property " + label0 + " " + words[0] + " " + var);
			println("set_property " + label1 + " " + words[0] + " " + var);
			}
			
		}
	
		else if (words[0].equals("assign")) {
			String label = words[1];
			String var = words[2];
			Property property = vars.get(var);
			
			if (property == null){
				println("assign failed "+label+" "+var);
			}
			else {
				property.label = label;

				println("assign " + label + " " + property.name + " " + property.label + " " + var);
				
				bounded_variables.add(label);
			}
		}
	}

	static void println(String line) {
		TextCanvas.textCanvas.println(line);
	}

	static public void unify() {
		HashSet<Assignment> consumed = new HashSet();
		HashSet<Visual> assigned = new HashSet();
		
		for (Visual visual : MainPanel.mainPanel.visuals) {
			Assignment assignment = get_assignment_by_tag(visual.tags);
			if (assignment != null) {
				visual.label = assignment.object_label; 
				merge_properties(visual.properties, assignment.properties);
				consumed.add(assignment);
				assigned.add(visual);
				System.out.println("unify by tag "+visual+" "+assignment);
			}
		}
		for (Visual visual : MainPanel.mainPanel.visuals) {
			if (assigned.contains(visual))
				continue;
			
			Assignment assignment = get_assignment_by_type(visual.get_name());
			if (consumed.contains(assignment))
					continue;
			if (assignment != null) {
					visual.label = assignment.object_label; 
					merge_properties(visual.properties, assignment.properties);
					
					consumed.add(assignment);
					assigned.add(visual);
				}
			
		}

		Analyzer.analyze_gaps();// stick.length -> node-node label
	}

	static public Assignment get_assignment_by_tag(ArrayList<String> tags) {
		for (Assignment assignment : assignments.values()) {
			if (is_one_of(assignment.object_label, tags))
				return assignment;
			for (Property property : assignment.properties) {
				System.out.println(""+property.name+" "+property.label);
				if (is_one_of(property.label, tags))
					return assignment;
			}
		}
		return null;
	}

	static public Assignment get_assignment_by_type(String name) {
		for (Assignment assignment : assignments.values()) {
			if (assignment.object_name.equals(name))
				return assignment;
		}
		return null;
	}

	static public void merge_properties(ArrayList<Property> visual_properties,
			ArrayList<Property> assignment_properties) {
		for (Property assignment_property : assignment_properties) {
			Property visual_property = find_property_by_name(assignment_property.name, visual_properties);
			if (visual_property != null) {
				visual_property.label = assignment_property.label;
			} else {
				visual_properties.add(assignment_property);
			}
		}
	}

	static public Property find_property_by_name(String name, ArrayList<Property> properties) {
		for (Property property : properties) {
			if (property.name.equals(name))
				return property;
		}
		return null;
	}

	static boolean is_one_of(String word, String[] keywords) {
		for (String keyword : keywords) {
			if (word.equals(keyword)) {
				return true;
			}
		}
		return false;
	}

	static boolean is_one_of(String word, ArrayList<String> keywords) {
		for (String keyword : keywords) {
			if (word.equals(keyword)) {
				return true;
			}
		}
		return false;
	}

}

class Assignment {
	String object_name;
	String object_label;
	ArrayList<Property> properties = new ArrayList();

	Assignment(String _object_name, String _object_label) {
		object_name = _object_name;
		object_label = _object_label;
	}
}
