package physics;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;


class MainPanel extends Panel implements ButtonsInterface{

	MainPanel(){
		MouseDispatcher mouseDispatcher = new MouseDispatcher();
		addMouseListener(mouseDispatcher);
		addMouseMotionListener(mouseDispatcher);
		addKeyListener(mouseDispatcher);
		
	}
	
	
	
	ArrayList<Visual> visuals = new ArrayList();
	ArrayList<Connection> connections = new ArrayList();
	ArrayList<Bucket> buckets = new ArrayList();
	HashMap<Node, HashMap<Node, String>> gap_labels = new HashMap<Node, HashMap<Node, String>>();
	
	//HashSet<String> constants = new HashSet();
	
	ArrayList<Mass> masses(){
		ArrayList<Mass> masses = new ArrayList();
		for(Visual visual : visuals){
			if (visual instanceof Mass)
				masses.add((Mass) visual);
		}
		return masses;
	}
	ArrayList<Wall> walls(){
		ArrayList<Wall> walls = new ArrayList();
		for(Visual visual : visuals){
			if (visual instanceof Wall)
				walls.add((Wall) visual);
		}
		return walls;
	}
	ArrayList<EdgeVisual> wires_and_walls_and_arrows(){
		ArrayList<EdgeVisual> walls = new ArrayList();
		for(Visual visual : visuals){
			if (visual instanceof Wall || visual instanceof Wire  || visual instanceof Arrow   || visual instanceof Spring)
				walls.add((EdgeVisual) visual);
		}
		return walls;
	}
	ArrayList<Wire> wires(){
		ArrayList<Wire> wires = new ArrayList();
		for(Visual visual : visuals){
			if (visual instanceof Wire)
				wires.add((Wire) visual);
		}
		return wires;
	}
	ArrayList<EdgeVisual> edgeVisuals(){
		ArrayList<EdgeVisual> edgeVisuals = new ArrayList();
		for(Visual visual : visuals){
			if (visual instanceof EdgeVisual)
				edgeVisuals.add((EdgeVisual) visual);
		}
		return edgeVisuals;
	}
	
	
	Visual selected_visual = null;
			
	Tool current_tool = new DrawStick();

	public void initializeButtons(Buttons buttons){
		//buttons.addButton("equations");
		buttons.addButton("clear");
		buttons.addButton("delete");
		//buttons.addButton("maxima");
	}
	public void buttonPressed(String label){
		if (label.equals("maxima")){
			//Analyzer.analyze();
			Maxima.run();
		}
		if (label.equals("equations")){
			//Analyzer.analyze();
			Core.equations();
		}
		if (label.equals("delete")){
			if (selected_visual != null)
				visuals.remove(selected_visual);
			selected_visual = null;
		}
		if (label.equals("clear")){
			visuals.clear();
			this.selected_visual = null;
			textCanvas.clear();
		}
		repaint();
	}	
	
	
	
	
	/*
	void solve(){
		println("--- force balance for each element ---");
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (! (visual instanceof Tag)){
				String line = visual.get_equation();
				if (line.length()>0)
					println(line);
			}
		}
		println("--- force balance for each connection ---");
		for(Connection connection : MainPanel.mainPanel.connections){
			
			String line_x = "";
			String line_y = "";
			for(Node node : connection.nodes){
				if (line_x.length()>0){
					line_x += " + ";
					line_y += " + ";
				}
				line_x += node.force_name_x;
				line_y += node.force_name_y;
			}
			line_x += " = 0";
			line_y += " = 0";
			
			println(line_x);
			println(line_y);
		}

	}
	*/
	
	void println(String line){
		textCanvas.println(line);
	}
	
	
	Image buffered_image = null;
	public void update(Graphics g) {
		paint(g);
	}
	
	public void paint(Graphics g) {

		
		{
			//double buffering
			if (buffered_image == null || buffered_image.getHeight(null) != this.getHeight() || buffered_image.getWidth(null) != this.getWidth())
				buffered_image = createImage(getSize().width, getSize().height);
			Graphics bg = buffered_image.getGraphics();

			// clear
			bg.setColor(Color.white);
			bg.fillRect(0,0, getSize().width, getSize().height);

			bg.setFont(new Font("Sans Serif", Font.PLAIN, 20));
			
			
			((Graphics2D)bg).setStroke(new BasicStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
			((Graphics2D)bg).setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
                    RenderingHints.VALUE_ANTIALIAS_ON);
			
			// paint code!
			for(int i=0; i<strokes.size(); i++){
				Stroke stroke = (Stroke) strokes.elementAt(i);
				stroke.paint(bg);
			}
			
			for(Visual visual : visuals){
				visual.paint(bg);
			}
			
			if (selected_visual != null)
				selected_visual.paint_bbox(bg);
			
			current_tool.paint(bg);


			
			
			//double buffering
			bg.dispose();
      
			g.drawImage(buffered_image,0,0,this);

			
		}
		
	}
	
	

   
   
	
	
	
	public void paint_stroke(Graphics g, Vector stroke){
		for(int i=0; i<stroke.size()-1; i++){
			Point p = (Point) stroke.elementAt(i);
			Point q = (Point) stroke.elementAt(i+1);
			g.drawLine(p.x, p.y, q.x, q.y);
		}
	}
	
	
	
	

	
	
	
	
	
	//
	// event handling
	//
	Vector strokes = new Vector();
	Stroke stroke = null;
	public void start_stroke(Point p){
		stroke = new Stroke();
		extend_stroke(p);
	}
	public void extend_stroke(Point p){
		stroke.addPoint(p);
		repaint();
	}
	public void finish_stroke(Point p){
		strokes.addElement(stroke);
	}
	
	
	

	
	
	
	
	
	

	
	
	
	
   
	
	public int operation_status;
	public static final int NONE = 0;
	public static final int DRAWING = 1;
   
	//   
	// central event dispatcher
	//
	public class MouseDispatcher 
		extends MouseAdapter implements MouseMotionListener, KeyListener{

  		public void mouseMoved(MouseEvent e) {
  		}
  		public void mousePressed(MouseEvent e) {
			Point p = e.getPoint();
			boolean right_button = (e.getModifiers() & e.BUTTON3_MASK) != 0;
			
			operation_status = DRAWING;
			//start_stroke(p);
			current_tool.start_dragging(e);
			
			repaint();

		}
  		public void mouseDragged(MouseEvent e) {
			Point p = e.getPoint();
			
			switch(operation_status){
			case DRAWING:
				current_tool.dragging(e);
				//extend_stroke(p);
				break;
			}
			
			repaint();
			
   		}
		public void mouseReleased(MouseEvent e) {
			Point p = e.getPoint();
			
			switch(operation_status){
			case DRAWING:
				current_tool.finish_dragging();
				
				Analyzer.analyze();
				break;
			}
			
			repaint();

		}
		public void mouseClicked(MouseEvent e){
		}
		  public void keyPressed(KeyEvent e){
		  }

		  public void keyReleased(KeyEvent e){
		  }

		  public void keyTyped(KeyEvent e){
			  //System.out.println(""+((int)e.getKeyChar())+" "+KeyEvent.VK_BACK_SPACE+" "+KeyEvent.VK_DELETE);
			  if (selected_visual != null){
				  if (((int)e.getKeyChar()) == KeyEvent.VK_BACK_SPACE ){
					  if (selected_visual.label.length() > 0){
						  selected_visual.label = selected_visual.label.substring(0, selected_visual.label.length()-1);
					  }
				  }
				  else if (((int)e.getKeyChar()) == KeyEvent.VK_DELETE ){
					  if (selected_visual != null)
							visuals.remove(selected_visual);
						selected_visual = null;
				  }
				  else
					  selected_visual.label+=e.getKeyChar();
				  //selected_visual.set_label(""+e.getKeyChar());
				  Analyzer.analyze();
			  }
			  repaint();
		  }
    }
	
	
	
	

	public static MainPanel mainPanel;
    public static JFrame frame;
    public static TextCanvas textCanvas;
	public static void main(String args[]) {
		frame = new JFrame("test.txt");
		frame.addWindowListener(
		  new WindowAdapter(){
			public void windowClosing(WindowEvent e) {System.exit(0);}}
			);
		frame.addKeyListener(
		  new KeyAdapter(){
		    public void keyPress(KeyEvent e) {
				if (e.getKeyCode() == e.VK_C)
					System.exit(0);}}
	        );

		
		/*
	    JSplitPane splitpane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitpane.setTopComponent(create_top_panel());
		splitpane.setBottomComponent(create_bottom_panel());
		splitpane.setDividerLocation(550);
		
	    frame.getContentPane().add(splitpane, BorderLayout.CENTER);
	    */
		
	    frame.getContentPane().add(create_top_panel());

	    
		frame.setMenuBar(new MyMenuBar());
	    
		frame.setSize(1024, 768);
		frame.setVisible(true);
    }
	
	static JSplitPane create_top_panel(){
	    JSplitPane splitpane = new JSplitPane();

	    frame.getContentPane().add(splitpane, BorderLayout.CENTER);
	    
	    splitpane.setDividerSize(5);
	    splitpane.setDividerLocation(500);
	    
		JPanel left_panel = new JPanel();
		left_panel.setLayout(new BorderLayout());
		mainPanel = new MainPanel();
		left_panel.add("Center", mainPanel);
		left_panel.add("South",new Buttons(mainPanel));
		left_panel.add("West",new ToolPanel());
		splitpane.setLeftComponent(left_panel);
		
		
		JPanel right_panel = new JPanel();
		right_panel.setLayout(new BorderLayout());
		textCanvas = new TextCanvas();
		//textCanvas = new InputCanvas();
		right_panel.add("Center", new JScrollPane(textCanvas));
		right_panel.add("South",new Buttons(textCanvas));
		splitpane.setRightComponent(right_panel);
	    
	    
	    splitpane.setDividerSize(5);
	    splitpane.setDividerLocation(500);

	    return splitpane;
	}
	static JPanel create_bottom_panel(){
	    JPanel panel = new JPanel();

	    ConsoleCanvas textCanvas = new ConsoleCanvas();
	    
	    panel.setLayout(new BorderLayout());
		panel.add("Center", new JScrollPane(textCanvas));
		panel.add("South",new Buttons(textCanvas));
	    
	    
	    return panel;
	}
}







