package physics;

import java.io.*;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

public class Maple {
	public static void main(String[] args){
	//public static void run(){
		//solve("[x+y=0, x-y=a*b], [x,y]");
		ArrayList<String> equations = new ArrayList();
		equations.add("x+y=a");
		equations.add("x-y=b");
		
		ArrayList<String> variables = new ArrayList();
		variables.add("x");
		variables.add("y");
		//variables.add("a");

		String[] solutions = solve(equations, variables);
		
		for(String solution : solutions)
			System.out.println(""+solution);
	}
	
	// equations = "x+y=0", "x-y=2", ...
	// variables = "x", "y", ...
	// input = "[x+y=0, x-y=a*b], [x,y]"
	public static String[] solve(ArrayList<String> equations, ArrayList<String> variables){
		
		
		String input = "";

		input += "{";
		for(String equation : equations){
			
			input += equation + ",\n";
		}
		input = input.substring(0, input.length()-2);	// remove last ","
		input += "}";
		
		input += ",{";
		for(String variable : variables)
			input += variable + ",";
		input = input.substring(0, input.length()-1);	// remove last ","
		input += "}";
		
		
		write_input(input);

		System.out.println("sent to maple : "+input);

		execute();
		
		String output = read_output();
		
		System.out.println("returned from maple : "+output);
		
		output = output.trim();
		
		// \left\{ d={\frac {{\it m0}\,g}{{\it k1}}},{\it f1}={\it m0}\,g,{\it fx0}=0,{\it fy0}={\it m0}\,g \right\} 

		output = convert_latex(output).trim();
		/*
		output = output
				.replace("\\{", "{")
				.replace("\\}", "}")
				.replace("{", "")
				.replace("}", "")
				.replace("\\left", "")
				.replace("\\right", "")
				.replace("\\it", "")
				.replace("\\,", "*")
				.replace("\\frac", "/");
		*/
		
		System.out.println("result: "+output);
		//if (output.equals("{}")){
		//		return null;
		//}
			
		//output = output.substring(1,  output.length()-1);	// remove [[ ]]
		
		String[] solutions = output.split(",");
		
		for(int i=0; i<solutions.length; i++)
			solutions[i] = solutions[i].trim();
		
		return solutions;
	}
	static String convert_latex(String input){
		String output="";

		int[] index = {0};
		
		output = read(input, index);
		
		return output;
	}
	static String read(String input, int[] index){
		String output="";

		int L = input.length();

		//System.out.println("read "+input.substring(index[0], L));
		
		while(index[0] < L-1){
			String remaining_input = input.substring(index[0], L);
			
			if (remaining_input.startsWith("\\{")){
				index[0]+=2;
			}
			if (remaining_input.startsWith("\\}")){
				index[0]+=2;
			}
			else if (remaining_input.startsWith("{")){
				index[0]+=1;
				String suboutput = read(input, index);
				//output+= "("+suboutput+")";
				output+= suboutput;
			}
			else if (remaining_input.startsWith("}")){
				index[0]+=1;
				//System.out.println("return "+output);
				return output;
			}
			else if (remaining_input.startsWith("\\it ")){
				index[0]+=4;
			}
			else if (remaining_input.startsWith("\\_")){
				index[0]+=2;
				output+="_";
			}
			else if (remaining_input.startsWith("\\theta")){
				index[0]+=6;
				output+="theta";
			}
			else if (remaining_input.startsWith("\\sin")){
				index[0]+=5;
				output+="sin";
			}
			else if (remaining_input.startsWith("\\cos")){
				index[0]+=5;
				output+="cos";
			}
			else if (remaining_input.startsWith("\\left")){
				index[0]+=5;
			}
			else if (remaining_input.startsWith("\\right")){
				index[0]+=6;
			}
			else if (remaining_input.startsWith("\\,")){
				index[0]+=2;
				output+="*";
			}
			else if (remaining_input.startsWith("\\frac")){
				index[0]+=6;
				index[0]+=1;	// discard "{"
				String arg0 = read(input, index);
				index[0]+=1;	// discard "{"
				String arg1 = read(input, index);
				//System.out.println("arg0 " + arg0);
				//System.out.println("arg1 " + arg1);
				output += "("+arg0+")/("+arg1+")";
			}
			else if (remaining_input.startsWith("\\sqrt")){
				index[0]+=6;
				index[0]+=1;	// discard "{"
				String arg0 = read(input, index);
				output += "sqrt("+arg0+")";
			}
			else {
				output+=input.substring(index[0], index[0]+1);
				index[0]+=1;
			}
		}
		
		return output;
	}

	public static String  read_output(){
		String output = "";
	    try{
	        File file = new File("output.txt");

	        BufferedReader br = new BufferedReader(new FileReader(file));

	        String str;
	        while((str = br.readLine()) != null){
		        	  output += str;
		    }
	     
	        br.close();
	        
	      }catch(IOException e){
	        System.out.println(e);
	      }
	    
	    return output;
	}
	public static void write_input(String equation){
	    try{
	        File file = new File("input.txt");

	        BufferedWriter bw = new BufferedWriter(new FileWriter(file));

	        //bw.write("display2d:false$");
	        //bw.newLine();
	        
	        bw.write("interface(quiet=true);");
	        bw.newLine();
	        
	        bw.write("writeto(\"output.txt\");");
	        bw.newLine();
	        
	        //bw.write("solve("+equation+");");
	        bw.write("latex(simplify(solve("+equation+")));");
	        bw.newLine();

	        bw.write("writeto(terminal);");
	        bw.newLine();

	        //bw.write("quit();");
	        //bw.newLine();

	        bw.close();
	      }catch(IOException e){
	        System.out.println(e);
	      }
	}
	public static void execute(){
		ProcessBuilder pb = new ProcessBuilder(
				"cmd", "/c",
				"C:\\Program Files (x86)\\Maple 2015\\bin.win\\cmaple.exe"
				,"<", "input.txt", ">", "terminal.txt"
			);
		
		String current_directory = System.getProperty("user.dir");
		System.out.println(current_directory);
		
		File dir = new File(current_directory);
		pb.directory(dir);
		
		try{
			Process process = pb.start();
			int ret = process.waitFor();
			
			System.out.println("quit maple"+ret);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	
	public static void printInputStream(InputStream is) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(is));
		try {
			for (;;) {
			      if(br.ready()){
			    	  System.out.println(".");
			    	  String line = br.readLine();
			    	  System.out.println("..");
			    	  if (line == null) 
			    		  break;
			    	  System.out.println(line);
			      }
			      
			      Thread.sleep(1000*2);
			}
		} catch(Exception e){
			e.printStackTrace();
		}
		finally {
		
			br.close();
		}
	}
}
