package physics;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.io.*;
import java.util.ArrayList;

public class Mass extends Visual{
	public Vector2D v0;
	String label = "";
	String direction = null;
	
	public Mass(){
		super();

	}
	public Mass(Vector2D v0){
		super();

		this.v0 = v0;
		
	}
	public void set_default_properties(){
		properties.clear();
		
		set_property("weight", "m"+id, true, true);
		set_property("force_x", "fx"+id, false, false);
		set_property("force_y", "fy"+id, false, false);

		set_property("sfc", "sfc"+id, true, true);	// skin friction coefficient
		set_property("distance", "distance"+id, true, true);	// travel distance
	}
	
	public void save(BufferedWriter out){
		Record.write(out, "v0", v0);
	}
	public void load(BufferedReader in){
		v0 = Record.read_Vector2D(in);
	}
	
	
	
	public String get_default_name(){
		return "mass"+id;
	}
	public String get_name(){
		return "mass";
	}

	
	Vector2D prev_p = null;
	public void start_dragging(Vector2D p){
		prev_p = p;
	}
	public void dragging(Vector2D p){
		Vector2D vec = new Vector2D(prev_p, p);
		v0.add(vec);
		prev_p = p;
	}
	public void finish_dragging(){
		v0 = Tool.snap(v0);
		
}
	
	
	
	static final double r = Tool.d;///2;
	public void paint(Graphics g){
		g.setColor(Color.black);
		g.fillOval((int)(v0.x-r),  (int)(v0.y-r),  (int)(2*r), (int)(2*r));
		
		g.drawString(label, (int)v0.x+15, (int)v0.y+10);
	}
	public void paint_bbox(Graphics g){
		g.setColor(Color.red);
		g.drawOval((int)(v0.x-r-3),  (int)(v0.y-r-3),  (int)(2*r+6), (int)(2*r+6));
	
		
	}
	public double get_distance(Vector2D v){
		return Vector2D.distance(v0,  v)-10;
	}
	public void set_label(String label){
		this.label = label;
	}
	public ArrayList<Node> get_nodes(){
		ArrayList<Node> nodes = new ArrayList();
		Node node = new Node(this, get_variable("force_x") , get_variable("force_y"), v0, "ep1(id"+id+")");
		
		if (Acceleration.enabled){
			String[] vec_xy = get_acceleration_vector();
			
			System.out.println("acceleration_x = "+vec_xy[0]);
			System.out.println("acceleration_y = "+vec_xy[1]);
			
			//String[] vec_xy = Acceleration.get_vec_from_connected_wire_or_wall(this);
			node.acceleration_x = get_variable("acceleration") + " * (1)* " + vec_xy[0] ;
			node.acceleration_y = get_variable("acceleration") + " * (1)* " + vec_xy[1];
			
			
		}
		
		nodes.add(node);
		
		return nodes;
	}
	
	Friction friction;
	//Acceleration acceleration;
	public ArrayList<String> get_equations(){
		
		System.out.println("friction "+friction);
		if (friction != null){
				// static 
				
				String force_x = get_variable("force_x");
				String force_y = "( " + get_variable("force_y") + " - " + get_variable("weight") + " * g )";

				String normal_x = "cos(" + friction.normal + ")";
				String normal_y = "sin(" + friction.normal + ")";

				//String tangent_x = "cos(" + friction.tangent + ")";
				//String tangent_y = "sin(" + friction.tangent + ")";

				String[] vec_xy = get_acceleration_vector();

				
				String normal_force = "("+force_x + " * " + normal_x + " + " + force_y + " * " + normal_y+")";
				String tangent_force = "("+force_x + " * " + vec_xy[0] + " + " + force_y + " * " + vec_xy[1]+")";

				ArrayList<String> equations = new ArrayList();

				if (!Acceleration.enabled){
					// ÖC
					equations.add("" + get_variable("friction_force") + " = " + tangent_force);
					
					if (Acceleration.limiting_friction){
					
					equations.add("" + get_variable("friction_force_max") + " = " + get_variable("sfc") + " * ( "
							+ normal_force + " ) ");
					equations.add(
							"" + get_variable("friction_force_max") + " =  ( " + get_variable("friction_force") + " )");
					}
				}			
				else {
					// C
					equations.add("" + get_variable("friction_force") + " = " + get_variable("sfc") + " * ( "
							+ normal_force + " ) ");
					
					equations.add(get_variable("acceleration") + " * " + get_variable("weight") + " = - " +  tangent_force 
							+ " - " + get_variable("friction_force"));
					
					
					equations.addAll(get_motion_equations());

					
				}

				return equations;
		}
		if (Acceleration.enabled){
			ArrayList<String> equations = new ArrayList();
			
			//String[] vec_xy = (direction != null) ?
			//		Acceleration.get_vec_from_direction(this) :
			//		Acceleration.get_vec_from_connected_wire_or_wall(this);
			String[] vec_xy = get_acceleration_vector();
			
			
			String acc_x = get_variable("acceleration") + " * " + get_variable("weight")  + " * " +  vec_xy[0] ;
			String acc_y = get_variable("acceleration") + " * " + get_variable("weight")  + " * " +  vec_xy[1];
			
			equations.add(""+get_variable("force_x") + " + " + acc_x + " = 0 "); 
			equations.add(""+get_variable("force_y") + " + " + acc_y + " = " + get_variable("weight") + " * g"); 

			equations.addAll(get_motion_equations());
			
			return equations;
		}
		
		ArrayList<String> equations = new ArrayList();
		equations.add(""+get_variable("force_x") + " = 0 "); 
		equations.add(""+get_variable("force_y") + " = " + get_variable("weight") + " * g"); 
		return equations;
	}
	
	String[] get_acceleration_vector(){
		
		if (friction != null){
			String tangent_x = "cos(" + friction.tangent + ")";
			String tangent_y = "sin(" + friction.tangent + ")";
			return new String[]{tangent_x, tangent_y};
		}
		
		
		if (direction != null) {
			String[] vec = Acceleration.get_vec_from_direction(this);
			System.out.println("vec "+vec[0]+" "+vec[1]);
			return vec;
		}
		
		return Acceleration.get_vec_from_connected_wire_or_wall(this);
	}
	

	public ArrayList<String> get_motion_equations(){
		ArrayList<String> equations = new ArrayList();
		
		String a = get_variable("acceleration");
		String d = get_variable("distance");	// const

		String v = get_variable("velocity");	// velocity * velocity
		String t = get_variable("time");		// time * time
		
		String E = get_variable("kinetic_energy");
		String m = get_variable("weight");
		
		//equations.add("" + t + " = 2*" + d + "/" + a + "");
		//equations.add(""+ v + " = " + a +" * " + a + " * " +t);
		//equations.add(""+E + " = (1/2) * " + m + " * "+v);

		equations.add("" + t + " * " + t + " = 2*" + d + "/" + a + "");
		equations.add("" + v + " * " + v + " = 2*" + d + " * " + a + "");
		//equations.add(""+ v + " = " + a + " * " +t);
		equations.add(""+E + " = (1/2) * " + m + " * "+v+" * "+v );
		
		String wg = get_variable("work_of_gravity");		// 	{͌l
		equations.add(""+wg + " = " + m + " * g * " + d);
		
		if (friction != null){
			String wf = get_variable("work_of_friction");	
			String friction_force = get_variable("friction_force");
			equations.add(""+wf + " = - " + friction_force + " * " + d);
		}
		
		return equations;
	}
}
class Friction {
	String normal;
	String tangent;
	Friction(String normal, String tangent){
		this.tangent = tangent;
		this.normal = normal;
	}
	
	/*
	Vector2D normal;
	Vector2D tangent;
	Friction(Vector2D normal){
		this.normal = normal;
		this.tangent = Vector2D.rotate90(normal);
	}
	*/
}