package physics;

import java.io.*;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

import javax.swing.JLabel;
import javax.swing.JOptionPane;

public class Maxima {
	public static void run(){
		
		ArrayList<String> equations = new ArrayList();
		equations.add("x+y=a");
		equations.add("x-y=b");
		
		ArrayList<String> variables = new ArrayList();
		equations.add("x");
		equations.add("y");
		equations.add("a");

		String[] solutions = solve(equations, variables);
		
		for(String solution : solutions)
			System.out.println(""+solution);
	}
	
	// equations = "x+y=0", "x-y=2", ...
	// variables = "x", "y", ...
	// input = "[x+y=0, x-y=a*b], [x,y]"
	public static String[] solve(ArrayList<String> equations, ArrayList<String> variables){
		
		
		String input = "";

		input += "[";
		for(String equation : equations){
			
			input += equation + ",\n";
		}
		input = input.substring(0, input.length()-2);	// remove last ","
		input += "]";
		
		input += ",[";
		for(String variable : variables)
			input += variable + ",";
		input = input.substring(0, input.length()-1);	// remove last ","
		input += "]";
		
		
		write_input(input);

		System.out.println("sent to maxima:\n"+input);

		int success = execute();
		
		if (success != 0){
				JLabel label = new JLabel("Failed to run Maxima. Please set 'maxima.txt' appropriately.");
			    JOptionPane.showMessageDialog(MainPanel.frame, label);
			return null;
		}
		
		
		String output = read_output();
		
		System.out.println("returned from maxima:\n"+output);
		
		if (output.trim().equals("[]")){
			return null;
		}
			
		output = output.substring(2,  output.length()-2);	// remove [[ ]]
		
		String[] solutions = output.split(",");
		
		return solutions;
	}

	public static String  read_output(){
		String output = "";
	    try{
	        File file = new File("output.txt");

	        BufferedReader br = new BufferedReader(new FileReader(file));

	        String str;
	        boolean reading = false;
	        while((str = br.readLine()) != null){
	          if (str.startsWith("(%o2)")){
	        	  output += str.substring(6);
	        	  reading = true;
	          }
	          else if (str.startsWith("(%i")){
	        	  reading = false;
	          }
	          else if (reading){
	        	  output += str;
	          }
	        }

	        br.close();
	        
	      }catch(IOException e){
	        System.out.println(e);
	      }
	    
	    return output;
	}
	public static void write_input(String equation){
	    try{
	        File file = new File("input.txt");

	        BufferedWriter bw = new BufferedWriter(new FileWriter(file));

	        bw.write("display2d:false$");
	        bw.newLine();
	        
	        bw.write("solve("+equation+");");
	        bw.newLine();

	        bw.write("quit();");
	        bw.newLine();

	        bw.close();
	      }catch(IOException e){
	        System.out.println(e);
	      }
	}
	static String maxima_path = null;
	static String get_maxima_path(){
		if (maxima_path != null)
			return maxima_path;

	    try{
	        File file = new File("maxima.txt");

	        BufferedReader br = new BufferedReader(new FileReader(file));

	        String str;
	        boolean reading = false;
	        while((str = br.readLine()) != null){
	        	maxima_path = str;
	        	System.out.println("set maxima path = "+maxima_path);
	        	return maxima_path;
	        }

	        br.close();
	        
	      }catch(IOException e){
	        System.out.println(e);
	      }
	    
	    return null;

	}
	
	
	private static final String OS_NAME = System.getProperty("os.name").toLowerCase();
	
	public static int execute(){
		try{

			ProcessBuilder pb;
		
		if (OS_NAME.startsWith("windows"))
		pb= new ProcessBuilder(
				"cmd", "/c",
				get_maxima_path(),
				//"C:\\maxima-5.37.3\\bin\\maxima.bat",
				"<", "input.txt", ">", "output.txt"
			);
		else 
			pb= new ProcessBuilder(
					get_maxima_path(),
					"<", "input.txt", ">", "output.txt"
				);
			
		String current_directory = System.getProperty("user.dir");
		//System.out.println(current_directory);
		
		File dir = new File(current_directory);
		pb.directory(dir);
		
			Process process = pb.start();
			int ret = process.waitFor();
			

			return ret;
			//System.out.println("quit maxima"+ret);
		}
		catch(Exception e){
			e.printStackTrace();
			
			return 1;
		}
		
	}
	
	/*
	public static void run(){
		ProcessBuilder pb = new ProcessBuilder("C:\\maxima-5.37.3\\bin\\maxima.bat");
		pb.redirectErrorStream(true);
		try{
			Process process = pb.start();
			
			
			
			
			System.out.println("hello");
			
			InputStream is = process.getInputStream();	
			printInputStream(is);

			System.out.println("hello");
			
			process.waitFor(3, TimeUnit.SECONDS);

			System.out.println("hello");
			
			printInputStream(is);

		
			is.close();
	
			System.out.println("hello");
			
			boolean end = process.waitFor(2, TimeUnit.SECONDS); //10bŃ^CAEg
			
			System.out.println("hello");
			
			if (end) {
				System.out.println("I " + process.exitValue());
			} else {
				System.out.println("^CAEg");
				process.destroy(); // vZXI
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	*/
	public static void printInputStream(InputStream is) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(is));
		try {
			for (;;) {
			      if(br.ready()){
			    	  System.out.println(".");
			    	  String line = br.readLine();
			    	  System.out.println("..");
			    	  if (line == null) 
			    		  break;
			    	  System.out.println(line);
			      }
			      
			      Thread.sleep(1000*2);
			}
		} catch(Exception e){
			e.printStackTrace();
		}
		finally {
		
			br.close();
		}
	}
}
