package physics;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;

public class Pulley extends EdgeVisual {
	ArrayList<Wire> wires = new ArrayList();
	
	public Pulley(){
		super();

	}
	public Pulley(Vector2D v0, Vector2D v1){
		super(v0, v1);
		


	}
	public void set_default_properties(){
		properties.clear();
		set_property("tensforce", "PT"+id, false, true);
		
	}

	public String get_default_name(){
		return "pulley"+id;
	}
	
	
	public void update_wires(){
		wires.clear();
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual instanceof Wire){
				Wire wire = (Wire) visual;
				if (Vector2D.distance(wire.v0, v0) < R + 10){
					connect_wire(wire, wire.v0);
					wires.add(wire);
					System.out.println("connect wire v0 "+this+" "+wire);
				}
				else if (Vector2D.distance(wire.v1, v0) < R + 10){
					connect_wire(wire, wire.v1);
					wires.add(wire);
					System.out.println("connect wire v1"+this+" "+wire);
				}
			}
		}
	}
	public void connect_wire(Wire wire, Vector2D v){
		Vector2D base = (wire.v0 == v ) ? wire.v1 : wire.v0;
		
		Vector2D base_to_center = new Vector2D(base, v0);
		double base_to_center_length = base_to_center.length();
		double center_to_contact_length = R;
		double base_to_contact_length = Math.sqrt(base_to_center_length*base_to_center_length - center_to_contact_length*center_to_contact_length);
		double base_to_foot_length = base_to_contact_length * base_to_contact_length / base_to_center_length;
		Vector2D vec = Vector2D.normalize(base_to_center);
		Vector2D foot = Vector2D.add(base, Vector2D.multiply(vec, base_to_foot_length));
		double foot_to_contact_length = base_to_contact_length * center_to_contact_length / base_to_center_length;
		vec = Vector2D.rotate90(vec);
		if (Vector2D.dot_product(new Vector2D(foot, v), vec) < 0)
			vec = Vector2D.negate(vec);
		Vector2D contact = Vector2D.add(foot,  Vector2D.multiply(vec,  foot_to_contact_length));
		
		v.warp(contact);
		
	}
	
	
	
	public int R = 20;
	public void paint(Graphics g){
		g.setColor(Color.black);
		//g.drawLine((int)v0.x, (int)v0.y, (int)v1.x, (int)v1.y);
		paint_rotated_rectangle(g, v0, v1, 8);	

		
		g.drawOval((int)v0.x-R, (int)v0.y-R, R*2, R*2);
		int r = 5;
		g.setColor(Color.white);
		g.fillOval((int)v0.x-r, (int)v0.y-r, r*2, r*2);
		g.setColor(Color.black);
		g.drawOval((int)v0.x-r, (int)v0.y-r, r*2, r*2);
	}
	public void paint_bbox(Graphics g){
		g.setColor(Color.red);
		
		Vector2D x_vec = new Vector2D(v0, v1);
		x_vec.normalize();
		Vector2D y_vec = Vector2D.rotate90(x_vec);
		x_vec.multiply(3);
		y_vec.multiply(3);
		
		Vector2D c0 = Vector2D.subtract(Vector2D.subtract(v0,  x_vec), y_vec);
		Vector2D c1 = Vector2D.add(Vector2D.subtract(v0,  x_vec), y_vec);
		Vector2D c2 = Vector2D.add(Vector2D.add(v1,  x_vec), y_vec);
		Vector2D c3 = Vector2D.subtract(Vector2D.add(v1,  x_vec), y_vec);
		drawLine(g, c0, c1);
		drawLine(g, c1, c2);
		drawLine(g, c2, c3);
		drawLine(g, c3, c0);

		
		double r = Tool.d;
		g.drawOval((int)(v0.x-r), (int)(v0.y-r), (int)(r*2), (int)(r*2));
		g.drawOval((int)(v1.x-r), (int)(v1.y-r), (int)(r*2), (int)(r*2));
	}
	
	
	
	
	public ArrayList<Node> get_nodes(){
		ArrayList<Node> nodes = new ArrayList();

		System.out.println("pulley "+id+" "+wires.size());
		for(Wire wire : wires){
			Vector2D contact;
			Vector2D base;
			if (Vector2D.distance(v0,  wire.v0) < Vector2D.distance(v0,  wire.v1)){
				contact = wire.v0;
				base = wire.v1;
			}
			else {
				contact = wire.v1;
				base = wire.v0;
			}

			String force_x = get_variable("tensforce")+ " * "+negate(wire.vec_x(contact));
			String force_y = get_variable("tensforce")+ " * "+negate(wire.vec_y(contact));

			Node node =  new Node(this,  force_x, force_y, contact, "id"+id);
			
			
			if (Acceleration.enabled){
				String sign = "";
				if (Vector2D.cross_product(new Vector2D(v0, contact),new Vector2D(contact, base)) > 0)
					sign = "(-1) * ";
				
				//node.acceleration_x = " ( "+sign + get_variable("acceleration") + " - " + get_variable("accelerationw") + " ) * " + wire.vec_x() ;
				//node.acceleration_y = " ( "+sign + get_variable("acceleration") + " - " + get_variable("accelerationw") + " ) * " + wire.vec_y() ;

				node.acceleration_x = get_variable("acceleration_x") + " + "+ " ( "+sign + get_variable("acceleration_r")  + " ) * " + wire.vec_x(contact) ;
				node.acceleration_y = get_variable("acceleration_y") + " + "+ " ( "+sign + get_variable("acceleration_r")  + " ) * " + wire.vec_y(contact) ; 

				
			}
		
			
			
			nodes.add(node);
		}
		
		
		// 
		
		Node node0 = new Node(this, get_variable("tensforce")+ " * 2 * "+vec_x(v1), get_variable("tensforce")+ " * 2 * "+vec_y(v1), v1, "ep1(id"+id+")");
		if (Acceleration.enabled){
			// node0.acceleration_x = get_variable("accelerationw") + " * " + vec_x(v1);// + "/2";
			// node0.acceleration_y = get_variable("accelerationw") + " * " + vec_y(v1);// + "/2" ;
			node0.acceleration_x = get_variable("acceleration_x") + " * " + vec_x(v1);// + "/2";
			node0.acceleration_y = get_variable("acceleration_y") + " * " + vec_y(v1);// + "/2" ;
		}
		
		nodes.add(node0);
		
		return nodes;
	}
	
	public ArrayList<String> get_free_variables(){
		ArrayList<String> variables = new ArrayList();
		variables.add(get_variable("tensforce"));
		return variables;
	}
	
	
}
