package physics;

import java.io.*;
import java.util.*;

public class Record {
	
	static public void write(BufferedWriter out, String name, Vector2D v){
		try{
			out.write(" <"+name+">");
			out.write("("+v.x+","+v.y+")");	
			out.write("</"+name+">");
			out.newLine();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	static public void write(BufferedWriter out, String name, String string){
		try{
			out.write("<"+name+">");
			out.write(string);	
			out.write("</"+name+">");
			out.newLine();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	static public void write(BufferedWriter out, String name, double x){
		try{
			out.write("<"+name+">");
			out.write(""+x);	
			out.write("</"+name+">");
			out.newLine();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	static public Vector2D read_Vector2D(BufferedReader in){
		
		try{
			String line = in.readLine();
			String[] tokens = line.split("[ <>(),/]+");

			
			double x = Double.parseDouble(tokens[2]);
			double y = Double.parseDouble(tokens[3]);
			return new Vector2D(x,y);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	static public double read_double(BufferedReader in){
		
		try{
			String line = in.readLine();
			String[] tokens = line.split("[ <>(),/]+");
			
			double x = Double.parseDouble(tokens[2]);
			return x;
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		
		return 0;
	}
	static public String read_String(BufferedReader in){
		
		try{
			String line = in.readLine();
			String[] tokens = line.split("[ <>(),/]+");
			
			return tokens[2];
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		
		return "";
	}
	
	static public void save(String filename)  {
		
		
		try {
			BufferedWriter out = (new BufferedWriter(new FileWriter(filename)));

			String serialized = serialize();
			out.write(serialized);

			out.flush();
			out.close();
		}
		catch (IOException e) {
			System.err.print("IO Error");
			e.printStackTrace();
		}
	}
	static public String serialize(){
		
		try{
			StringWriter writer = new StringWriter();
		BufferedWriter out = (new BufferedWriter(writer));
		
		for(Visual visual : MainPanel.mainPanel.visuals){
			
			if (visual instanceof Tag && ((Tag)visual).label.length()==0)
				continue;
			
			String classname = visual.getClass().getName();
			out.write("<visual class=\"" + classname + "\">");
			out.newLine();

			visual.save(out);
			
			out.write("</visual>");
			out.newLine();
			
		}
		out.write("<text>");
		out.newLine();

		if (InputCanvas.inputCanvas != null)
			out.write(InputCanvas.inputCanvas.getText());
		else
			out.write(TextCanvas.textCanvas.saveText());
	
		out.write("</text>");
		out.newLine();
		
		out.flush();
		out.close();
		
		return (writer.getBuffer().substring(0));
		
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return null;
	}
	static public void load(String filename) {
		
		String loaded_text = "";
		
		try{
			
			ArrayList<Visual> visuals = new ArrayList();
			
			BufferedReader in = new BufferedReader(new FileReader(filename));
			String line;
			while ((line = in.readLine()) != null) {
				if (line.startsWith("<visual")){		// <visual class="Mass">
					String[] tokens = line.split("[< =\"]+");
					//for(String token : tokens)
					//	System.out.println("["+token+"]");
					String classname = tokens[3];
					Class myClass = Class.forName(classname);
					Visual visual = (Visual)myClass.newInstance();
					
					visual.load(in);
					
					in.readLine();	// "</visual>"
					
					visuals.add(visual);
				}
				else if (line.startsWith("<text>")){
					String text = "";
					while(true){
						line = in.readLine();
						if (line.equals("</text>"))
							break;
						text += line + "\n";
					}
					if (InputCanvas.inputCanvas != null)
						InputCanvas.inputCanvas.setText(text);
					else {
						loaded_text = text;
					}
				}
			
			}
			in.close();
						
			
			MainPanel.mainPanel.visuals = visuals;

			
			if (InputCanvas.inputCanvas == null){
				Analyzer.analyze();
				TextCanvas.textCanvas.loadText(loaded_text);
			}
			
		}
		catch (Exception e) {
			
			e.printStackTrace();
		}
		
		
		
	}
}
