package physics;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;

public class Spring extends EdgeVisual {
	//public Vector2D v0;
	//public Vector2D v1;
	public Spring(){
		super();

	}
	public Spring(Vector2D v0, Vector2D v1){
		super(v0, v1);
	}
	public void set_default_properties(){
		properties.clear();
		set_property("cnst_spring", "k"+id, true, true);
		set_property("growth", "d"+id, false, false);
		set_property("force", "f"+id, false, false);

	}
	public String get_default_name(){
		return "spring"+id;
	}
	public String get_name(){
		return "spring";
	}

	
	public void paint(Graphics g){
		g.setColor(Color.black);
		
		double l = Vector2D.distance(v0, v1);
		double d = 20;
		Vector2D x_vec = new Vector2D(v0, v1);
		x_vec.normalize();
		Vector2D y_vec = Vector2D.rotate90(x_vec);

		Vector2D prev_p = v0;
		
		double rx = 10;
		double ry = 15;
		int n = (int)(l/d);
		int steps = 32;
		for(int i=2; i<n-2; i++){
			for(int j=0; j<=steps; j++){
				double angle = Math.PI*2*j/steps;
				double x = rx * Math.cos(angle)+d*i+d*j/steps;
				double y = ry * Math.sin(angle);

				Vector2D p = Vector2D.add(v0, Vector2D.multiply(x_vec,  x), Vector2D.multiply(y_vec,  y));
				if (prev_p != null){
					drawLine(g, prev_p, p);
				}
				prev_p = p;
			}
		}
		drawLine(g, prev_p, v1);
	}
	public void paint_bbox(Graphics g){
		g.setColor(Color.red);
		
		Vector2D x_vec = new Vector2D(v0, v1);
		x_vec.normalize();
		Vector2D y_vec = Vector2D.rotate90(x_vec);
		x_vec.multiply(3);
		y_vec.multiply(18);
		
		Vector2D c0 = Vector2D.subtract(Vector2D.subtract(v0,  x_vec), y_vec);
		Vector2D c1 = Vector2D.add(Vector2D.subtract(v0,  x_vec), y_vec);
		Vector2D c2 = Vector2D.add(Vector2D.add(v1,  x_vec), y_vec);
		Vector2D c3 = Vector2D.subtract(Vector2D.add(v1,  x_vec), y_vec);
		drawLine(g, c0, c1);
		drawLine(g, c1, c2);
		drawLine(g, c2, c3);
		drawLine(g, c3, c0);
		
	}
	
	public ArrayList<Node> get_nodes(){
		// Vector2D direction = new Vector2D(v0, v1);
		// direction.normalize();
		
		ArrayList<Node> nodes = new ArrayList();
		nodes.add(new Node(this,   get_variable("force")+ " * "+vec_x(v0), get_variable("force")+ " * "+vec_y(v0), v0, "ep1(id"+id+")"));
		nodes.add(new Node(this,   get_variable("force")+ " * "+vec_x(v1), get_variable("force")+ " * "+vec_y(v1), v1, "ep2(id"+id+")"));
		return nodes;
	}

	public ArrayList<String> get_equations(){
		ArrayList<String> equations = new ArrayList();
		equations.add(""+get_variable("force") + " = " + get_variable("cnst_spring") + " * "+get_variable("growth")); 
		return equations;
	}
	public ArrayList<String> get_free_variables(){
		ArrayList<String> variables = new ArrayList();
		variables.add(get_variable("force"));
		// variables.add(get_variable("stiffness"));
		variables.add(get_variable("growth"));
		return variables;
	}
}
