package physics;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

public class Stick extends EdgeVisual {

	public Stick(){
		super();

	}
	public Stick(Vector2D v0, Vector2D v1){
		super(v0,v1);
		
	}
	public void set_default_properties(){
		properties.clear();

		set_property("weight", "weight"+id, true, true);	// ftHgŏdȂH
		set_property("force0_x", "force0_x"+id, false, false);
		set_property("force0_y", "force0_y"+id, false, false);
		set_property("force1_x", "force1_x"+id, false, false);
		set_property("force1_y", "force1_y"+id, false, false);

		set_property("length", "length"+id, true, true);
	}
	public String get_default_name(){
		return "stick"+id;
	}
	public String get_name(){
		return "stick";
	}

	public void paint_property(Graphics g, Point offset){
	}
	public void paint(Graphics g){
		g.setColor(Color.black);
		paint_rotated_rectangle(g, v0, v1, 5);	
	
	}
	public void paint_bbox(Graphics g){
		g.setColor(Color.red);
		
		Vector2D x_vec = new Vector2D(v0, v1);
		x_vec.normalize();
		Vector2D y_vec = Vector2D.rotate90(x_vec);
		x_vec.multiply(3);
		y_vec.multiply(8);
		
		Vector2D c0 = Vector2D.subtract(Vector2D.subtract(v0,  x_vec), y_vec);
		Vector2D c1 = Vector2D.add(Vector2D.subtract(v0,  x_vec), y_vec);
		Vector2D c2 = Vector2D.add(Vector2D.add(v1,  x_vec), y_vec);
		Vector2D c3 = Vector2D.subtract(Vector2D.add(v1,  x_vec), y_vec);
		drawLine(g, c0, c1);
		drawLine(g, c1, c2);
		drawLine(g, c2, c3);
		drawLine(g, c3, c0);
		
	}


	public Node start_node;
	public Node end_node;
	public int node_index = 2;
	public void clear_nodes(){
		node_index = 2;
	}
	public Node get_node(Vector2D position){
		//return new Node(this, "sublength", get_variable("force2_x"), get_variable("force2_y"), position, "ep2(id"+id+")");
		String force_x = "force"+node_index+"_x";
		String force_y = "force"+node_index+"_y";
		
		
		Node node = new Node(this, "sublength"+node_index, get_variable(force_x), get_variable(force_y), position, "ep2(id"+id+")");
		
		node_index++;
		
		return node;
	}
	public ArrayList<Node> get_nodes(){
		start_node = new Node(this, "0", get_variable("force0_x"), get_variable("force0_y"), v0, "ep0(id"+id+")");
		end_node = new Node(this, "length", get_variable("force1_x"), get_variable("force1_y"), v1, "ep1(id"+id+")");

		ArrayList<Node> nodes = new ArrayList();
		nodes.add(start_node);
		nodes.add(end_node);
		return nodes;
		
	}
	
	
	
	public ArrayList<String> gap_labels = new ArrayList();
	public ArrayList<String> get_equations(){
		
		ArrayList<Node> nodes = new ArrayList();
		nodes.add(start_node);
		nodes.add(end_node);
		for(Connection connection : MainPanel.mainPanel.connections){
			for(Node node : connection.nodes){
				if (node.visual == this && !nodes.contains(node)){
					nodes.add(node);
				}
			}
		}

		
		Collections.sort(nodes, new NodeComparator(v0));
		Node start_node = nodes.get(0);
		Node end_node = nodes.get(nodes.size()-1);

		
		HashMap<Node,String> distance_from_start = new HashMap();

		
		// @0 | L0 | L1 | L2 | L3 | ....
		distance_from_start.put(start_node,  "0");
		for(int i=1; i<nodes.size()-1; i++){
			Node node0 = nodes.get(i);
			distance_from_start.put(node0,  "L"+(i-1));
			
			this.set_property("L"+(i-1), "L"+(i-1), false, true);
		}
		
		distance_from_start.put(end_node,  get_property("length").label);
		Analyzer.add_gap_label(start_node,  end_node, get_property("length").label);
		
		
		// xƂ̑Ήt
		gap_labels.clear();
		ArrayList<String> gap_equations = new ArrayList();
		for(int i=0; i<nodes.size()-1; i++){
			Node node0 = nodes.get(i);
			for(int j=i+1; j<nodes.size(); j++){
				Node node1 = nodes.get(j);
				String gap_label = Analyzer.get_gap_label(node0,  node1);
				if (gap_label != null){
					gap_labels.add(gap_label);
					
					String gap_equation = gap_label + " = " + distance_from_start.get(node1) + " - " + distance_from_start.get(node0);;
					gap_equations.add(gap_equation);
				}
			}
		}
		
		

		// rotation (tangent) direction
		String rx = "(-sin("+get_variable("angle")+"))";
		String ry = "(+cos("+get_variable("angle")+"))";
		
		String line_x = "";
		String line_y = "";
		String moment = "";
		for(Node node : nodes){
			
			if (line_x.length() > 0)
				line_x += " + ";
			line_x += node.force_name_x;
			
			if (line_y.length() > 0)
				line_y += " + ";
			line_y += node.force_name_y;
			
			if (node == start_node)
				continue;

			if (moment.length() > 0)
				moment += " + ";
			

			
			moment += node.force_name_x+" * "+"("+distance_from_start.get(node)+")"+" * "+rx+" + "
					+ node.force_name_y+" * "+"("+distance_from_start.get(node)+")"+" * "+ry;
		}
		line_y += " - "+get_variable("weight") + " * g";

		{
			moment += " - "+get_variable("weight")+ " * g * "+ "("+distance_from_start.get(end_node)+")"+" * "+ry+ " / 2 ";
		}
		
		line_x +=" = 0";
		line_y +=" = 0";
		moment +=" = 0";

		
		ArrayList<String> equations = new ArrayList();

		equations.add(line_x);
		equations.add(line_y);
		equations.add(moment);
		
		equations.addAll(gap_equations);
		
		return equations;
	}
	
	/*
	// ÂŁ@gap_label 
	public ArrayList<String> _get_equations(){
		
		ArrayList<Node> nodes = new ArrayList();
		nodes.add(start_node);
		nodes.add(end_node);
		for(Connection connection : MainPanel.mainPanel.connections){
			for(Node node : connection.nodes){
				if (node.visual == this && !nodes.contains(node)){
					nodes.add(node);
				}
			}
		}

		
		Collections.sort(nodes, new NodeComparator(v0));
		Node start_node = nodes.get(0);
		Node end_node = nodes.get(nodes.size()-1);

		

		
		// L0 | L1 | L2 | L3 | ....
		for(int i=0; i<nodes.size()-1; i++){
			Node node0 = nodes.get(i);
			Node node1 = nodes.get(i+1);
			String label = Analyzer.get_gap_label(node0, node1);
			if (label == null)
				Analyzer.add_gap_label(node0, node1, "L"+i);
		}
		// "L0+L1+L2 = L"
		ArrayList<String> gap_equations = new ArrayList();
		for(int i=0; i<nodes.size()-2; i++){
			Node node0 = nodes.get(i);
			for(int j=i+2; j<nodes.size(); j++){
				Node node1 = nodes.get(j);
				String gap_label = Analyzer.get_gap_label(node0,  node1);
				if (gap_label != null){
					String gap_equation = gap_label + " = ";
					for(int k=i; k<j; k++){
						Node node2 = nodes.get(k);
						Node node3 = nodes.get(k+1);
						gap_equation += "+"+Analyzer.get_gap_label(node2, node3);
					}
					gap_equations.add(gap_equation);
				}
			}
		}
		// 0; 0+L0; 0+L0+L1; 0+L0+L1+L2; ....
		Analyzer.add_gap_label(nodes.get(0), nodes.get(0),  "0");
		for(int i=1; i<nodes.size(); i++){
			Node node0 = nodes.get(0);
			Node prev_node1 = nodes.get(i-1);
			Node node1 = nodes.get(i);
			String label = Analyzer.get_gap_label(node0, node1);
			if (label == null)
				Analyzer.add_gap_label(node0, node1, Analyzer.get_gap_label(node0, prev_node1) + "+"+Analyzer.get_gap_label(prev_node1,  node1));
		}
		
		
		
		
		String line_x = "";
		String line_y = "";
		String moment = "";
		for(Node node : nodes){
			if (line_x.length() > 0)
				line_x += " + ";
			line_x += node.force_name_x;
			
			if (line_y.length() > 0)
				line_y += " + ";
			line_y += node.force_name_y;
			
			if (moment.length() > 0)
				moment += " + ";
			
			//Vector2D vec = new Vector2D(v0, node.position);
			//vec.normalize();
			//vec = new Vector2D(-vec.y, vec.x);
			//moment += node.force_name_x+" * "+"("+Analyzer.get_gap_label(start_node,  node)+")"+" * "+round(vec.x)+" + "
			//		+ node.force_name_y+" * "+"("+Analyzer.get_gap_label(start_node,  node)+")"+" * "+round(vec.y);
			moment += node.force_name_x+" * "+"("+Analyzer.get_gap_label(start_node,  node)+")"+" * "+vec_x()+" + "
					+ node.force_name_y+" * "+"("+Analyzer.get_gap_label(start_node,  node)+")"+" * "+vec_y();
		}
		line_y += " - "+get_variable("weight") + " * g";

		{
			Vector2D vec = new Vector2D(v0, Vector2D.mid_point(v0,  v1));
			vec.multiply(1.0/Vector2D.distance(v0, v1));
			vec = new Vector2D(-vec.y, vec.x);
			moment += " - "+get_variable("weight")+ " * g * "+ "("+Analyzer.get_gap_label(start_node,  end_node)+")"+" * "+vec_y();
		}
		
		line_x +=" = 0";
		line_y +=" = 0";
		moment +=" = 0";

		
		ArrayList<String> equations = new ArrayList();

		equations.add(line_x);
		equations.add(line_y);
		equations.add(moment);
		//equations.addAll(gap_equations);
		
		return equations;
	}
	*/
	
	public class NodeComparator implements Comparator<Node> {
		Vector2D base;
		NodeComparator(Vector2D base){
			this.base = base;
		}
	    //r\bhif[^NXr-1, 0, 1Ԃ悤ɋLqj
	    public int compare(Node a, Node b) {
	        double no1 = Vector2D.distance(base,  a.position);
	        double no2 = Vector2D.distance(base,  b.position);

	        //ƎЈԍ̏Ń\[g
	        if (no1 > no2) {
	            return 1;

	        } else if (no1 == no2) {
	            return 0;

	        } else {
	            return -1;

	        }
	    }

	}
	/*
	public ArrayList<String> get_free_variables(){
		ArrayList<String> variables = new ArrayList();


		ArrayList<Node> nodes = new ArrayList();
		for(Connection connection : MainPanel.mainPanel.connections){
			for(Node node : connection.nodes){
				if (node.visual == this){
					nodes.add(node);
				}
			}
		}
		
		for(Node node : nodes){
			variables.add(node.force_name_x);
			variables.add(node.force_name_y);
			
			Property property_length = get_property(node.length);
			if (property_length!=null && !property_length.constant)
				variables.add(property_length.label);
		}
		
		//variables.add(get_variable("length"));
		//variables.add(get_variable("sublength"));
		

		//variables.add(get_variable("mass"));

		return variables;
	}
	*/
}
