package physics;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

import javax.imageio.ImageIO;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.*;
import javax.swing.text.rtf.RTFEditorKit;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.text.StyledDocument;

public class TextCanvas extends JTextPane implements  KeyListener, CaretListener,  ButtonsInterface{
	
	MutableAttributeSet attribute = new SimpleAttributeSet();
	
	public static TextCanvas textCanvas;
	
	static HashMap<String, String> property_names = new HashMap();
	static HashMap<String, String> entity_names = new HashMap();
	
	static public HashSet<String> bounded_variables = new HashSet();

	
	TextCanvas() {
	
		textCanvas = this;
		
		addKeyListener(this);
		addCaretListener(this);
		
		StyleContext style = new StyleContext();
		document = new TextCanvasDocument(style, this);
		this.setDocument(document);

		attribute = new SimpleAttributeSet();
		StyleConstants.setFontSize(attribute,20);
		
		this.setCharacterAttributes(attribute, true);
		
		
		/*
		KeyStroke ctrlV = KeyStroke.getKeyStroke(KeyEvent.VK_V, KeyEvent.CTRL_MASK);
        this.getInputMap().put(ctrlV,"pasteKey");
        this.getActionMap().put("pasteKey", pasteKey);
        */
		
        property_names.put("weight",  "weight");
        property_names.put("mass",  "weight");

        property_names.put("cnst_spring",  "cnst_spring");
        property_names.put("stiffness",  "cnst_spring");

        property_names.put("growth",  "growth");
        property_names.put("length",  "length");
        
        property_names.put("sfc", "sfc");
        property_names.put("friction", "sfc");
        property_names.put("friction_coefficient", "sfc");
        
        property_names.put("tensforce", "tensforce");
        property_names.put("tension", "tensforce");
        
        property_names.put("rel_force", "rel_force");
        property_names.put("force", "rel_force");

        
        entity_names.put("line", "line");
        entity_names.put("wire", "line");
        entity_names.put("string", "line");
        entity_names.put("cable", "line");
        
        entity_names.put("stick", "stick");
        entity_names.put("bar", "stick");
        
	}
	void loadText(String lines){
		
		
		this.setText("");

		BufferedReader reader = new BufferedReader(new StringReader(lines));
		while(true){
			try{
			String line = reader.readLine();
			line = line.trim();
			System.out.println("load "+line);
			if (line== null)
				break;
			document.insertString(document.getLength(), line+"\n", attribute);
			
			execute(line);
			}
			catch(Exception e){
				break;
			}
		}
		
	}
	String saveText(){
		String output = "";
		
		String lines = this.getText();
		BufferedReader reader = new BufferedReader(new StringReader(lines));
		while(true){
			try{
			String line = reader.readLine();
			line = line.trim();
			System.out.println("load "+line);
			if (line== null)
				break;
			if (line.startsWith(">"))
				continue;
			output += line +"\n";
			}
			catch(Exception e){
				break;
			}
		}
		return output;
	}
	static String get_property_name(String alias){
		String property_name = property_names.get(alias);
		if (property_name == null)
			return alias;
		return property_name;
	}
	static String get_entity_name(String alias){
		String entity_name = entity_names.get(alias);
		if (entity_name == null)
			return alias;
		return entity_name;
	}
	
	
	public void initializeButtons(Buttons buttons){
		buttons.addButton("clear");
		//buttons.addButton("pasteLF");
		//buttons.addButton("assignments");
		//buttons.addButton("tags");
		
		//buttons.addButton("enable acceleration");
		//buttons.addButton("properties");
		//buttons.addButton("equations");
	}
	public void buttonPressed(String label){
		if (label.equals("clear")){
			//MainPanel.mainPanel.visuals.clear();
			//MainPanel.mainPanel.selected_visual = null;
			clear();
		}
		if (label.equals("enable acceleration")){
			Acceleration.enabled = true;
			println("enable acceleration");
		}
		if (label.equals("pasteLF")){
			pasteLF();
		}
		if (label.equals("assignments")){
			this.show_assignments();
		}
		if (label.equals("tags")){
			show_tags();
		}
		if (label.equals("properties")){
			show_properties();
		}
		if (label.equals("equations")){
			Core.equations();
		}
		repaint();
	}	
	
	

	
	
	 Action pasteKey = new AbstractAction() {
         public void actionPerformed(ActionEvent e) {
             //paste();
        	 pasteLF();
         }
       };
      public void pasteLF(){
          String text = get_text_from_clipboard();
          
          LF.process_lines(text);
          
          println("unify -----------------------");

          Analyzer.analyze();
          
          LF.unify();
     	 
          show_properties();
      }
      
     String get_text_from_clipboard(){
    	 Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
    	 Transferable object = clipboard.getContents(null);
    	 String str = "";
    	 try {
    	   str = (String)object.getTransferData(DataFlavor.stringFlavor);
    	 } catch(UnsupportedFlavorException e){
    	   e.printStackTrace();
    	 } catch (IOException e) {
    	   e.printStackTrace();
    	 }
    	 return str;
     }
       
     public void show_properties(){
    	 println("properties -----------------------");

			for(Visual visual : MainPanel.mainPanel.visuals){
				if (!(visual instanceof Tag) && !(visual instanceof Gap) && !(visual instanceof Arc)){
					println(""+visual);
					println("  name="+visual.get_name());
					println("  id="+visual.id);
					println("  label="+visual.label);
					show_properties(visual);
				}
			}
     }
     public void show_tags(){
    	 println("tags -----------------------");

			for(Visual visual : MainPanel.mainPanel.visuals){
				if (!(visual instanceof Tag) && !(visual instanceof Gap) && !(visual instanceof Arc)){
					println(""+visual);
					for(String tag : visual.tags){
						println("  "+tag);
					}
				}
			}
     }

     public void show_assignments(){
    	 println("assignments -----------------------");
       	 for(Assignment assignment : LF.assignments.values()){
   			 println(""+assignment.object_name+" "+assignment.object_label);
       		 for(Property property : assignment.properties){
       			 println("  "+property.name+" "+property.label);
       		 }
    	 }
    	 
     }

       
	public void clear(){
		try{
		document.remove(0,  document.getLength());
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	public void println(String line){
		try{
		document.insertString(document.getLength(), "> "+line+"\n", attribute);
		//document.insertString(document.getLength(), ""+line+"\n", attribute);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	public void	keyPressed(KeyEvent e){

	}
	
	String get_prev_last_line(){
		ArrayList<String> lines = get_lines(this.getText());
		return lines.get(lines.size()-1);
		/*
		String text = this.getText();
		String last_line = "";
		String prev_last_line = "";
		for(int i=0; i<text.length(); i++){
			String c = text.substring(i, i+1);
			if (c.equals("\n")){
				prev_last_line = last_line;
				last_line = "";
			}
			else
				last_line +=c;
		}
		return prev_last_line;
		*/
	}
	ArrayList<String> get_lines(String string){
		ArrayList<String> lines = new ArrayList();
		StringTokenizer st = new StringTokenizer(string, "\n");
		while(st.hasMoreTokens()){
			String line = st.nextToken().trim();
			if (line.length() > 0)
				lines.add(line);
		}
		return lines;
		
	}
	String get_last_line(){
		String text = this.getText();
		String last_line = "";
		for(int i=0; i<text.length(); i++){
			String c = text.substring(i, i+1);
			if (c.equals("\n"))
				last_line = "";
			else
				last_line +=c;
		}
		return last_line;
	}
	String get_before_last_line(){
		String text = this.getText();
		String last_line = get_last_line();
		return text.substring(0,  text.length()-last_line.length());
	
	}
	public void	keyReleased(KeyEvent e){
		boolean shift = (e.getModifiers() & e.SHIFT_MASK) != 0;
		boolean ctrl = (e.getModifiers() & e.CTRL_MASK) != 0;
		
		if (e.getKeyCode() == e.VK_ENTER){
			execute(get_prev_last_line().trim());
		}
		else if (e.getKeyCode() == e.VK_SPACE){
		}
		repaint();

		

	}
	public void	keyTyped(KeyEvent e) {
		
	}
	void repaint_all(){
		repaint(0,0, this.getWidth(), this.getHeight());
		
		
	}
	
	TextCanvasDocument document;
	public int selstart,selend;
	
	public void caretUpdate(CaretEvent ev) {
		selstart = this.getSelectionStart();
		selend = this.getSelectionEnd();
		repaint_all();
		
		MainPanel.mainPanel.selected_visual = null;
		MainPanel.mainPanel.repaint();
	}
	
	


	public void execute(String line){
		String[] words = line.split(" ", 0);

		System.out.println("execute ["+line+"]");
		
		if (line.startsWith("enable acceleration")){
			//Acceleration.enable_acceleration();
			Acceleration.enabled = true;
			println("enable acceleration");
			return;
		}
		else if (line.startsWith("limiting friction")){
			//Acceleration.enable_acceleration();
			Acceleration.limiting_friction = true;
			println("limiting friction");
			return;
		}
		else if (line.startsWith("assignments")){
			show_assignments();
			return;
		}
		else if (line.startsWith("properties")){
			Analyzer.analyze();
			show_properties();
			return;
		}
		else if (line.startsWith("tags")){
			show_tags();
			return;
		}
		else if (line.startsWith("equations")){
			//println("equations -----------------------");
			Analyzer.analyze();
			Core.equations();
			return;
		}
		/*
		else if (line.startsWith("answer equation of motion of")){
			String[] keywords = line.split(" ");
			String entity_name = keywords[5];
			
			Visual visual = get_visual_by_tag(entity_name);
			String eom = visual.get_variable("eom");
			String weight = visual.get_variable("weight");
			String accelration = visual.get_variable("acceleration");
			
			
			Analyzer.analyze();
			
			String result = Core.solve(eom);
			result = result.split("[ =]+")[1];
			
			println(""+weight+" * "+accelration + " = " + result);
			
			
			return;
		}
		*/
		else if (line.startsWith("set direction of")){
			String[] keywords = line.split(" ");
			String entity_name = keywords[3];
			
			
			Visual visual = get_visual(entity_name); 	// tag or default_name;

			
			
			if (!( visual instanceof Mass))
				return;
			
			Mass mass = (Mass) visual;
			
			mass.direction = keywords[4];
			
			println("set direction of "+visual.id+" "+mass.direction);
		}
		/*
		else if (line.startsWith("show forces of")){
			String[] keywords = line.split(" ");
			String entity_name = keywords[3];
			
			Visual visual = get_visual(entity_name);

			if (! (visual instanceof Stick))
				return;
			
			Stick stick = (Stick) visual;
			
			Analyzer.analyze();

			String result = Core.solve(null);
			
			String lines[] = result.split("\n");
			for(String l : lines){
				System.out.println(""+l);
			}
			for(Node node : stick.get_nodes()){
				System.out.println(""+node.force_name_x+" "+node.force_name_y);
			}
		}
		*/
		else if (line.startsWith("select equation of motion of")){
			String[] keywords = line.split(" ");
			String entity_name = keywords[5];
			
			Visual visual = get_visual(entity_name);

			
			String eom = visual.get_variable("eom");
			String weight = visual.get_variable("weight");
			String accelration = visual.get_variable("acceleration");
			
			String left = weight+" * "+accelration;
		
			Analyzer.analyze();

			Answers.prepare_select_equation_of_motion_of(left);

			String result = Core.solve(null);
			result = Answers.select_equation_of_motion_of(result, left);
			
			println(result);
		}
		
		else if (contains(words, "solve") && contains(words, "of") ){
			String[] keywords = line.split(" ");
			String property_name = get_property_name(keywords[1]);
			String entity_name = keywords[3];

			Visual visual = get_visual(entity_name);
			String target = visual.get_variable(property_name);
					
			Analyzer.analyze();

			String result = Core.solve(target);
			result = result.split("=")[1].trim(); // maple
			println(result);
			
			return;
		}
	
		else if (line.equals("select answer")){
			Analyzer.analyze();
			String result = Core.solve(null);
			result = Answers.select_answer(result);
			println(result);
			
		}
		else if (contains(words, "solve")){
			Analyzer.analyze();
			String result = Core.solve(words[1]);
			println(result);
			return;
		}
		else if (contains(words, ":") && contains(words, "=")){	// a0 : M * a = M * g - T
			String variable1 = line.split(":")[0].trim();
			String equation = line.split(":")[1].trim();
			
			String left = " ( " + equation.split("=")[0].trim()+ " ) ";
			String right = " ( " + equation.split("=")[1].trim()+ " ) ";

			Answers.answers.add(new Answer(variable1, left, right));
			
			return;
		}
		else if (contains(words, "=")){	// a0 : M * a = M * g - T
			
			String variable = line.split("=")[0].trim();
			String equation = line.split("=")[1].trim();

			Core.additional_variables.add(variable);
			Core.additional_equations.add(line);
			
			return;
		}

		
		if (contains(words, "of") && contains(words, "is")){
			
			String[] delimiters = {"of", "is"};
			String[] keywords = divide(words, delimiters);
			
			String property_name = get_property_name(keywords[0]);

			String entity_name = keywords[1];
			String value = keywords[2];

			
			ArrayList<Visual> visuals = get_visuals(entity_name); 	// tag or default_name;
			
			for(Visual visual : visuals){
				visual.set_property(property_name, value);
			
				println("set_property "+visual.label+" "+property_name+" "+value);
			}
			
			bounded_variables.add(value);
			/*
			Visual visual = get_visual(entity_name); 	// tag or default_name;
			
			visual.set_property(property_name, value);
			
			println("set_property "+visual.label+" "+property_name+" "+value);
			*/
			return;
		}
		
		println("?");
	}
	Visual get_visual(String entity_name){
		Visual visual = get_visual_by_tag(entity_name);
		if (visual == null)
				visual = get_visual_by_default_name(get_entity_name(entity_name));
		return visual;
	}
	ArrayList<Visual> get_visuals(String entity_name){
		ArrayList<Visual> visuals = get_visuals_by_tag(entity_name);
		if (visuals.size()==0)
				visuals = get_visuals_by_default_name(get_entity_name(entity_name));
		return visuals;
	}
	
	Visual find_visual_by_property_value(String property_value){
		for(Visual visual : MainPanel.mainPanel.visuals){
			for(Property property : visual.properties){
				if (property.label.equals(property_value)){
					return visual;
				}
			}
		}
		return null;
	}
	Property find_property_by_property_value(Visual visual, String property_value){
		for(Property property : visual.properties){
			if (property.label.equals(property_value)){
				return property;
			}
		}
		return null;
	}
	void show_properties(Visual visual){
		for(Property property : visual.properties){
			println("  "+property.name + " : "+ property.label+" "+property.constant);
		}
		
	}
	String[] divide(String[] words, String[] delimiters){
		String[] results = new String[words.length - delimiters.length]; 
		int index = 0;
		for(String word : words){
			if (contains(delimiters, word))
				continue;
			results[index++] = word;
		}
		return results;
	}
	boolean contains(String[] words, String keyword){
		for(String word : words){
			if (word.equals(keyword))
				return true;
		}
		return false;
	}
	
	Visual get_visual_by_tag(String label){
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual.contains_tag(label))
				return visual;
		}
		return null;
	}
	Visual get_visual_by_default_name(String label){
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual.get_default_name().startsWith(label))
					return visual;
		}
		return null;
	}
	ArrayList<Visual> get_visuals_by_tag(String label){
		ArrayList<Visual> visuals = new ArrayList<Visual>();
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual.contains_tag(label))
				visuals.add(visual);
		}
		return visuals;
	}
	ArrayList<Visual> get_visuals_by_default_name(String label){
		ArrayList<Visual> visuals = new ArrayList<Visual>();
		for(Visual visual : MainPanel.mainPanel.visuals){
			if (visual.get_default_name().startsWith(label))
				visuals.add(visual);
		}
		return visuals;
	}
}
